# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vmc.draas.model.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class AbstractEntity(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'id': 'id',
                            'version': 'version',
                            'created': 'created',
                            'user_id': 'user_id',
                            'user_name': 'user_name',
                            'updated': 'updated',
                            'updated_by_user_id': 'updated_by_user_id',
                            'updated_by_user_name': 'updated_by_user_name',
                            }

    def __init__(self,
                 id=None,
                 version=None,
                 created=None,
                 user_id=None,
                 user_name=None,
                 updated=None,
                 updated_by_user_id=None,
                 updated_by_user_name=None,
                ):
        """
        :type  id: :class:`str`
        :param id: 
        :type  version: :class:`long`
        :param version: Version of this entity format: int32
        :type  created: :class:`datetime.datetime`
        :param created: 
        :type  user_id: :class:`str`
        :param user_id: User id that last updated this record
        :type  user_name: :class:`str`
        :param user_name: User name that last updated this record
        :type  updated: :class:`datetime.datetime`
        :param updated: 
        :type  updated_by_user_id: :class:`str`
        :param updated_by_user_id: User id that last updated this record
        :type  updated_by_user_name: :class:`str`
        :param updated_by_user_name: User name that last updated this record
        """
        self.id = id
        self.version = version
        self.created = created
        self.user_id = user_id
        self.user_name = user_name
        self.updated = updated
        self.updated_by_user_id = updated_by_user_id
        self.updated_by_user_name = updated_by_user_name
        VapiStruct.__init__(self)


AbstractEntity._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.abstract_entity', {
        'id': type.StringType(),
        'version': type.IntegerType(),
        'created': type.DateTimeType(),
        'user_id': type.StringType(),
        'user_name': type.StringType(),
        'updated': type.DateTimeType(),
        'updated_by_user_id': type.StringType(),
        'updated_by_user_name': type.StringType(),
    },
    AbstractEntity,
    False,
    None))



class ActivateSiteRecoveryConfig(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'srm_extension_key_suffix': 'srm_extension_key_suffix',
                            }

    def __init__(self,
                 srm_extension_key_suffix=None,
                ):
        """
        :type  srm_extension_key_suffix: :class:`str` or ``None``
        :param srm_extension_key_suffix: Optional custom extension key suffix for SRM. If not specified,
            default extension key will be used. The custom extension suffix
            must contain 13 characters or less, be composed of letters,
            numbers, ., -, and _ characters. The extension suffix must begin
            and end with a letter or number. The suffix is appended to
            com.vmware.vcDr- to form the full extension key.
        """
        self.srm_extension_key_suffix = srm_extension_key_suffix
        VapiStruct.__init__(self)


ActivateSiteRecoveryConfig._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.activate_site_recovery_config', {
        'srm_extension_key_suffix': type.OptionalType(type.StringType()),
    },
    ActivateSiteRecoveryConfig,
    False,
    None))



class BuildVersion(VapiStruct):
    """
    The build version.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'version': 'version',
                            'build_number': 'build_number',
                            }

    def __init__(self,
                 version=None,
                 build_number=None,
                ):
        """
        :type  version: :class:`str` or ``None``
        :param version: 
        :type  build_number: :class:`str` or ``None``
        :param build_number: 
        """
        self.version = version
        self.build_number = build_number
        VapiStruct.__init__(self)


BuildVersion._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.build_version', {
        'version': type.OptionalType(type.StringType()),
        'build_number': type.OptionalType(type.StringType()),
    },
    BuildVersion,
    False,
    None))



class DatastoreEntity(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'id': 'id',
                            'name': 'name',
                            'datastore_type': 'datastore_type',
                            'instance_type': 'instance_type',
                            'number_of_hosts': 'number_of_hosts',
                            'replications_number': 'replications_number',
                            'total_datastore_size': 'total_datastore_size',
                            'replica_disks': 'replica_disks',
                            }

    def __init__(self,
                 id=None,
                 name=None,
                 datastore_type=None,
                 instance_type=None,
                 number_of_hosts=None,
                 replications_number=None,
                 total_datastore_size=None,
                 replica_disks=None,
                ):
        """
        :type  id: :class:`str` or ``None``
        :param id: 
        :type  name: :class:`str` or ``None``
        :param name: 
        :type  datastore_type: :class:`str` or ``None``
        :param datastore_type: 
        :type  instance_type: :class:`str` or ``None``
        :param instance_type: 
        :type  number_of_hosts: :class:`float` or ``None``
        :param number_of_hosts: 
        :type  replications_number: :class:`float` or ``None``
        :param replications_number: 
        :type  total_datastore_size: :class:`float` or ``None``
        :param total_datastore_size: 
        :type  replica_disks: :class:`list` of :class:`ReplicaDisk` or ``None``
        :param replica_disks: 
        """
        self.id = id
        self.name = name
        self.datastore_type = datastore_type
        self.instance_type = instance_type
        self.number_of_hosts = number_of_hosts
        self.replications_number = replications_number
        self.total_datastore_size = total_datastore_size
        self.replica_disks = replica_disks
        VapiStruct.__init__(self)


DatastoreEntity._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.datastore_entity', {
        'id': type.OptionalType(type.StringType()),
        'name': type.OptionalType(type.StringType()),
        'datastore_type': type.OptionalType(type.StringType()),
        'instance_type': type.OptionalType(type.StringType()),
        'number_of_hosts': type.OptionalType(type.DoubleType()),
        'replications_number': type.OptionalType(type.DoubleType()),
        'total_datastore_size': type.OptionalType(type.DoubleType()),
        'replica_disks': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'ReplicaDisk'))),
    },
    DatastoreEntity,
    False,
    None))



class DeleteConfigInternal(VapiStruct):
    """
    Deactivate Site Recovery or delete Site Recovery node configuration

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'force': 'force',
                            'ticket': 'ticket',
                            'confirmation_code': 'confirmation_code',
                            }

    def __init__(self,
                 force=None,
                 ticket=None,
                 confirmation_code=None,
                ):
        """
        :type  force: :class:`bool` or ``None``
        :param force: If = 'true', will deactivate site recovery forcefully.
        :type  ticket: :class:`str` or ``None``
        :param ticket: CSSD/CSCM ticket number, such as CSSD-xxxx/CSCM-xxxx
        :type  confirmation_code: :class:`str` or ``None``
        :param confirmation_code: The confirmation code generated in prepare delete operation.
        """
        self.force = force
        self.ticket = ticket
        self.confirmation_code = confirmation_code
        VapiStruct.__init__(self)


DeleteConfigInternal._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.delete_config_internal', {
        'force': type.OptionalType(type.BooleanType()),
        'ticket': type.OptionalType(type.StringType()),
        'confirmation_code': type.OptionalType(type.StringType()),
    },
    DeleteConfigInternal,
    False,
    None))



class ErrorResponse(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'error_code': 'error_code',
                            'error_messages': 'error_messages',
                            'status': 'status',
                            'path': 'path',
                            'retryable': 'retryable',
                            }

    def __init__(self,
                 error_code=None,
                 error_messages=None,
                 status=None,
                 path=None,
                 retryable=None,
                ):
        """
        :type  error_code: :class:`str`
        :param error_code: unique error code
        :type  error_messages: :class:`list` of :class:`str`
        :param error_messages: localized error messages
        :type  status: :class:`long`
        :param status: HTTP status code
        :type  path: :class:`str`
        :param path: Originating request URI
        :type  retryable: :class:`bool`
        :param retryable: If true, client should retry operation
        """
        self.error_code = error_code
        self.error_messages = error_messages
        self.status = status
        self.path = path
        self.retryable = retryable
        VapiStruct.__init__(self)


ErrorResponse._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.error_response', {
        'error_code': type.StringType(),
        'error_messages': type.ListType(type.StringType()),
        'status': type.IntegerType(),
        'path': type.StringType(),
        'retryable': type.BooleanType(),
    },
    ErrorResponse,
    False,
    None))



class Fault(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'message': 'message',
                            'cause': 'cause',
                            }

    def __init__(self,
                 message=None,
                 cause=None,
                ):
        """
        :type  message: :class:`str` or ``None``
        :param message: 
        :type  cause: :class:`vmware.vapi.struct.VapiStruct` or ``None``
        :param cause: 
        """
        self.message = message
        self.cause = cause
        VapiStruct.__init__(self)


Fault._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.fault', {
        'message': type.OptionalType(type.StringType()),
        'cause': type.OptionalType(type.DynamicStructType('vmware.vapi.dynamic_struct', {}, VapiStruct)),
    },
    Fault,
    False,
    None))



class HmsIssueInfo(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'target_object_name': 'target_object_name',
                            'target_object_mo_id': 'target_object_mo_id',
                            'triggered_time': 'triggered_time',
                            'issue_type': 'issue_type',
                            'fault': 'fault',
                            'severity': 'severity',
                            }

    def __init__(self,
                 target_object_name=None,
                 target_object_mo_id=None,
                 triggered_time=None,
                 issue_type=None,
                 fault=None,
                 severity=None,
                ):
        """
        :type  target_object_name: :class:`str` or ``None``
        :param target_object_name: 
        :type  target_object_mo_id: :class:`str`
        :param target_object_mo_id: 
        :type  triggered_time: :class:`datetime.datetime`
        :param triggered_time: 
        :type  issue_type: :class:`str`
        :param issue_type: 
        :type  fault: :class:`Fault` or ``None``
        :param fault: 
        :type  severity: :class:`str`
        :param severity: 
        """
        self.target_object_name = target_object_name
        self.target_object_mo_id = target_object_mo_id
        self.triggered_time = triggered_time
        self.issue_type = issue_type
        self.fault = fault
        self.severity = severity
        VapiStruct.__init__(self)


HmsIssueInfo._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.hms_issue_info', {
        'target_object_name': type.OptionalType(type.StringType()),
        'target_object_mo_id': type.StringType(),
        'triggered_time': type.DateTimeType(),
        'issue_type': type.StringType(),
        'fault': type.OptionalType(type.ReferenceType(__name__, 'Fault')),
        'severity': type.StringType(),
    },
    HmsIssueInfo,
    False,
    None))



class HmsReplicationIssueInfo(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'target_object_name': 'target_object_name',
                            'target_object_mo_id': 'target_object_mo_id',
                            'triggered_time': 'triggered_time',
                            'issue_type': 'issue_type',
                            'fault': 'fault',
                            'severity': 'severity',
                            'source_site_uuid': 'source_site_uuid',
                            'destination_site_uuid': 'destination_site_uuid',
                            'direction': 'direction',
                            }

    def __init__(self,
                 target_object_name=None,
                 target_object_mo_id=None,
                 triggered_time=None,
                 issue_type=None,
                 fault=None,
                 severity=None,
                 source_site_uuid=None,
                 destination_site_uuid=None,
                 direction=None,
                ):
        """
        :type  target_object_name: :class:`str` or ``None``
        :param target_object_name: 
        :type  target_object_mo_id: :class:`str`
        :param target_object_mo_id: 
        :type  triggered_time: :class:`datetime.datetime`
        :param triggered_time: 
        :type  issue_type: :class:`str`
        :param issue_type: 
        :type  fault: :class:`Fault` or ``None``
        :param fault: 
        :type  severity: :class:`str`
        :param severity: 
        :type  source_site_uuid: :class:`str` or ``None``
        :param source_site_uuid: 
        :type  destination_site_uuid: :class:`str` or ``None``
        :param destination_site_uuid: 
        :type  direction: :class:`str` or ``None``
        :param direction: 
        """
        self.target_object_name = target_object_name
        self.target_object_mo_id = target_object_mo_id
        self.triggered_time = triggered_time
        self.issue_type = issue_type
        self.fault = fault
        self.severity = severity
        self.source_site_uuid = source_site_uuid
        self.destination_site_uuid = destination_site_uuid
        self.direction = direction
        VapiStruct.__init__(self)


HmsReplicationIssueInfo._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.hms_replication_issue_info', {
        'target_object_name': type.OptionalType(type.StringType()),
        'target_object_mo_id': type.StringType(),
        'triggered_time': type.DateTimeType(),
        'issue_type': type.StringType(),
        'fault': type.OptionalType(type.ReferenceType(__name__, 'Fault')),
        'severity': type.StringType(),
        'source_site_uuid': type.OptionalType(type.StringType()),
        'destination_site_uuid': type.OptionalType(type.StringType()),
        'direction': type.OptionalType(type.StringType()),
    },
    HmsReplicationIssueInfo,
    False,
    None))



class HmsSiteIssueInfo(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'target_object_name': 'target_object_name',
                            'target_object_mo_id': 'target_object_mo_id',
                            'triggered_time': 'triggered_time',
                            'issue_type': 'issue_type',
                            'fault': 'fault',
                            'severity': 'severity',
                            }

    def __init__(self,
                 target_object_name=None,
                 target_object_mo_id=None,
                 triggered_time=None,
                 issue_type=None,
                 fault=None,
                 severity=None,
                ):
        """
        :type  target_object_name: :class:`str` or ``None``
        :param target_object_name: 
        :type  target_object_mo_id: :class:`str`
        :param target_object_mo_id: 
        :type  triggered_time: :class:`datetime.datetime`
        :param triggered_time: 
        :type  issue_type: :class:`str`
        :param issue_type: 
        :type  fault: :class:`Fault` or ``None``
        :param fault: 
        :type  severity: :class:`str`
        :param severity: 
        """
        self.target_object_name = target_object_name
        self.target_object_mo_id = target_object_mo_id
        self.triggered_time = triggered_time
        self.issue_type = issue_type
        self.fault = fault
        self.severity = severity
        VapiStruct.__init__(self)


HmsSiteIssueInfo._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.hms_site_issue_info', {
        'target_object_name': type.OptionalType(type.StringType()),
        'target_object_mo_id': type.StringType(),
        'triggered_time': type.DateTimeType(),
        'issue_type': type.StringType(),
        'fault': type.OptionalType(type.ReferenceType(__name__, 'Fault')),
        'severity': type.StringType(),
    },
    HmsSiteIssueInfo,
    False,
    None))



class ProvisionSrmConfig(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'srm_extension_key_suffix': 'srm_extension_key_suffix',
                            }

    def __init__(self,
                 srm_extension_key_suffix=None,
                ):
        """
        :type  srm_extension_key_suffix: :class:`str` or ``None``
        :param srm_extension_key_suffix: Optional custom extension key suffix for SRM. If not specified,
            default extension key will be used.
        """
        self.srm_extension_key_suffix = srm_extension_key_suffix
        VapiStruct.__init__(self)


ProvisionSrmConfig._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.provision_srm_config', {
        'srm_extension_key_suffix': type.OptionalType(type.StringType()),
    },
    ProvisionSrmConfig,
    False,
    None))



class ReplicaData(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'datastore_entities': 'datastore_entities',
                            }

    def __init__(self,
                 datastore_entities=None,
                ):
        """
        :type  datastore_entities: :class:`list` of :class:`DatastoreEntity` or ``None``
        :param datastore_entities: DatastoreEntity
        """
        self.datastore_entities = datastore_entities
        VapiStruct.__init__(self)


ReplicaData._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.replica_data', {
        'datastore_entities': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'DatastoreEntity'))),
    },
    ReplicaData,
    False,
    None))



class ReplicaDisk(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'disk_id': 'disk_id',
                            'collection_id': 'collection_id',
                            'name': 'name',
                            'datastore_mo_id': 'datastore_mo_id',
                            'space_requirement': 'space_requirement',
                            'movable': 'movable',
                            'datastores_for_single_host_move': 'datastores_for_single_host_move',
                            }

    def __init__(self,
                 disk_id=None,
                 collection_id=None,
                 name=None,
                 datastore_mo_id=None,
                 space_requirement=None,
                 movable=None,
                 datastores_for_single_host_move=None,
                ):
        """
        :type  disk_id: :class:`str` or ``None``
        :param disk_id: 
        :type  collection_id: :class:`str` or ``None``
        :param collection_id: 
        :type  name: :class:`str` or ``None``
        :param name: 
        :type  datastore_mo_id: :class:`str` or ``None``
        :param datastore_mo_id: 
        :type  space_requirement: :class:`float` or ``None``
        :param space_requirement: 
        :type  movable: :class:`bool` or ``None``
        :param movable: 
        :type  datastores_for_single_host_move: :class:`list` of :class:`vmware.vapi.struct.VapiStruct` or ``None``
        :param datastores_for_single_host_move: 
        """
        self.disk_id = disk_id
        self.collection_id = collection_id
        self.name = name
        self.datastore_mo_id = datastore_mo_id
        self.space_requirement = space_requirement
        self.movable = movable
        self.datastores_for_single_host_move = datastores_for_single_host_move
        VapiStruct.__init__(self)


ReplicaDisk._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.replica_disk', {
        'disk_id': type.OptionalType(type.StringType()),
        'collection_id': type.OptionalType(type.StringType()),
        'name': type.OptionalType(type.StringType()),
        'datastore_mo_id': type.OptionalType(type.StringType()),
        'space_requirement': type.OptionalType(type.DoubleType()),
        'movable': type.OptionalType(type.BooleanType()),
        'datastores_for_single_host_move': type.OptionalType(type.ListType(type.DynamicStructType('vmware.vapi.dynamic_struct', {}, VapiStruct))),
    },
    ReplicaDisk,
    False,
    None))



class ReplicaDiskCollection(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'collection_id': 'collection_id',
                            'name': 'name',
                            'placeholder_vm_mo_id': 'placeholder_vm_mo_id',
                            'disks': 'disks',
                            'generated': 'generated',
                            }

    def __init__(self,
                 collection_id=None,
                 name=None,
                 placeholder_vm_mo_id=None,
                 disks=None,
                 generated=None,
                ):
        """
        :type  collection_id: :class:`str` or ``None``
        :param collection_id: 
        :type  name: :class:`str` or ``None``
        :param name: 
        :type  placeholder_vm_mo_id: :class:`str` or ``None``
        :param placeholder_vm_mo_id: 
        :type  disks: :class:`list` of :class:`ReplicaDisk` or ``None``
        :param disks: 
        :type  generated: :class:`datetime.datetime` or ``None``
        :param generated: 
        """
        self.collection_id = collection_id
        self.name = name
        self.placeholder_vm_mo_id = placeholder_vm_mo_id
        self.disks = disks
        self.generated = generated
        VapiStruct.__init__(self)


ReplicaDiskCollection._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.replica_disk_collection', {
        'collection_id': type.OptionalType(type.StringType()),
        'name': type.OptionalType(type.StringType()),
        'placeholder_vm_mo_id': type.OptionalType(type.StringType()),
        'disks': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'ReplicaDisk'))),
        'generated': type.OptionalType(type.DateTimeType()),
    },
    ReplicaDiskCollection,
    False,
    None))



class ReplicationData(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'replication_name': 'replication_name',
                            'replication_uuid': 'replication_uuid',
                            'hbr_server_uuid': 'hbr_server_uuid',
                            'hbr_server_name': 'hbr_server_name',
                            'scale_out': 'scale_out',
                            }

    def __init__(self,
                 replication_name=None,
                 replication_uuid=None,
                 hbr_server_uuid=None,
                 hbr_server_name=None,
                 scale_out=None,
                ):
        """
        :type  replication_name: :class:`str` or ``None``
        :param replication_name: 
        :type  replication_uuid: :class:`str` or ``None``
        :param replication_uuid: 
        :type  hbr_server_uuid: :class:`str` or ``None``
        :param hbr_server_uuid: 
        :type  hbr_server_name: :class:`str` or ``None``
        :param hbr_server_name: 
        :type  scale_out: :class:`bool` or ``None``
        :param scale_out: 
        """
        self.replication_name = replication_name
        self.replication_uuid = replication_uuid
        self.hbr_server_uuid = hbr_server_uuid
        self.hbr_server_name = hbr_server_name
        self.scale_out = scale_out
        VapiStruct.__init__(self)


ReplicationData._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.replication_data', {
        'replication_name': type.OptionalType(type.StringType()),
        'replication_uuid': type.OptionalType(type.StringType()),
        'hbr_server_uuid': type.OptionalType(type.StringType()),
        'hbr_server_name': type.OptionalType(type.StringType()),
        'scale_out': type.OptionalType(type.BooleanType()),
    },
    ReplicationData,
    False,
    None))



class ReportPeriodConfig(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'report_start_day_inclusive': 'report_start_day_inclusive',
                            'report_end_day_inclusive': 'report_end_day_inclusive',
                            }

    def __init__(self,
                 report_start_day_inclusive=None,
                 report_end_day_inclusive=None,
                ):
        """
        :type  report_start_day_inclusive: :class:`datetime.datetime` or ``None``
        :param report_start_day_inclusive: 
        :type  report_end_day_inclusive: :class:`datetime.datetime` or ``None``
        :param report_end_day_inclusive: 
        """
        self.report_start_day_inclusive = report_start_day_inclusive
        self.report_end_day_inclusive = report_end_day_inclusive
        VapiStruct.__init__(self)


ReportPeriodConfig._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.report_period_config', {
        'report_start_day_inclusive': type.OptionalType(type.DateTimeType()),
        'report_end_day_inclusive': type.OptionalType(type.DateTimeType()),
    },
    ReportPeriodConfig,
    False,
    None))



class SiteRecovery(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """
    SITE_RECOVERY_STATE_ACTIVATING = "ACTIVATING"
    """


    """
    SITE_RECOVERY_STATE_ACTIVATED = "ACTIVATED"
    """


    """
    SITE_RECOVERY_STATE_DEACTIVATING = "DEACTIVATING"
    """


    """
    SITE_RECOVERY_STATE_DEACTIVATED = "DEACTIVATED"
    """


    """
    SITE_RECOVERY_STATE_FAILED = "FAILED"
    """


    """
    SITE_RECOVERY_STATE_CANCELED = "CANCELED"
    """


    """
    SITE_RECOVERY_STATE_DELETED = "DELETED"
    """


    """



    _canonical_to_pep_names = {
                            'id': 'id',
                            'version': 'version',
                            'created': 'created',
                            'user_id': 'user_id',
                            'user_name': 'user_name',
                            'updated': 'updated',
                            'updated_by_user_id': 'updated_by_user_id',
                            'updated_by_user_name': 'updated_by_user_name',
                            'sddc_id': 'sddc_id',
                            'site_recovery_state': 'site_recovery_state',
                            'draas_h5_url': 'draas_h5_url',
                            'vr_node': 'vr_node',
                            'srm_nodes': 'srm_nodes',
                            'vrs_nodes': 'vrs_nodes',
                            }

    def __init__(self,
                 id=None,
                 version=None,
                 created=None,
                 user_id=None,
                 user_name=None,
                 updated=None,
                 updated_by_user_id=None,
                 updated_by_user_name=None,
                 sddc_id=None,
                 site_recovery_state=None,
                 draas_h5_url=None,
                 vr_node=None,
                 srm_nodes=None,
                 vrs_nodes=None,
                ):
        """
        :type  id: :class:`str`
        :param id: 
        :type  version: :class:`long`
        :param version: Version of this entity format: int32
        :type  created: :class:`datetime.datetime`
        :param created: 
        :type  user_id: :class:`str`
        :param user_id: User id that last updated this record
        :type  user_name: :class:`str`
        :param user_name: User name that last updated this record
        :type  updated: :class:`datetime.datetime`
        :param updated: 
        :type  updated_by_user_id: :class:`str`
        :param updated_by_user_id: User id that last updated this record
        :type  updated_by_user_name: :class:`str`
        :param updated_by_user_name: User name that last updated this record
        :type  sddc_id: :class:`str` or ``None``
        :param sddc_id: 
        :type  site_recovery_state: :class:`str` or ``None``
        :param site_recovery_state: Possible values are: 
            
            * :attr:`SiteRecovery.SITE_RECOVERY_STATE_ACTIVATING`
            * :attr:`SiteRecovery.SITE_RECOVERY_STATE_ACTIVATED`
            * :attr:`SiteRecovery.SITE_RECOVERY_STATE_DEACTIVATING`
            * :attr:`SiteRecovery.SITE_RECOVERY_STATE_DEACTIVATED`
            * :attr:`SiteRecovery.SITE_RECOVERY_STATE_FAILED`
            * :attr:`SiteRecovery.SITE_RECOVERY_STATE_CANCELED`
            * :attr:`SiteRecovery.SITE_RECOVERY_STATE_DELETED`
        :type  draas_h5_url: :class:`str` or ``None``
        :param draas_h5_url: 
        :type  vr_node: :class:`SiteRecoveryNode` or ``None``
        :param vr_node: 
        :type  srm_nodes: :class:`list` of :class:`SrmNode` or ``None``
        :param srm_nodes: 
        :type  vrs_nodes: :class:`list` of :class:`SiteRecoveryNode` or ``None``
        :param vrs_nodes: 
        """
        self.id = id
        self.version = version
        self.created = created
        self.user_id = user_id
        self.user_name = user_name
        self.updated = updated
        self.updated_by_user_id = updated_by_user_id
        self.updated_by_user_name = updated_by_user_name
        self.sddc_id = sddc_id
        self.site_recovery_state = site_recovery_state
        self.draas_h5_url = draas_h5_url
        self.vr_node = vr_node
        self.srm_nodes = srm_nodes
        self.vrs_nodes = vrs_nodes
        VapiStruct.__init__(self)


SiteRecovery._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.site_recovery', {
        'id': type.StringType(),
        'version': type.IntegerType(),
        'created': type.DateTimeType(),
        'user_id': type.StringType(),
        'user_name': type.StringType(),
        'updated': type.DateTimeType(),
        'updated_by_user_id': type.StringType(),
        'updated_by_user_name': type.StringType(),
        'sddc_id': type.OptionalType(type.StringType()),
        'site_recovery_state': type.OptionalType(type.StringType()),
        'draas_h5_url': type.OptionalType(type.StringType()),
        'vr_node': type.OptionalType(type.ReferenceType(__name__, 'SiteRecoveryNode')),
        'srm_nodes': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'SrmNode'))),
        'vrs_nodes': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'SiteRecoveryNode'))),
    },
    SiteRecovery,
    False,
    None))



class SiteRecoveryNode(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """
    TYPE_VRMS = "VRMS"
    """


    """
    TYPE_SRM = "SRM"
    """


    """
    TYPE_VRS = "VRS"
    """


    """
    STATE_DEPLOYING = "DEPLOYING"
    """


    """
    STATE_PROVISIONED = "PROVISIONED"
    """


    """
    STATE_READY = "READY"
    """


    """
    STATE_DELETING = "DELETING"
    """


    """
    STATE_FAILED = "FAILED"
    """


    """
    STATE_CANCELED = "CANCELED"
    """


    """



    _canonical_to_pep_names = {
                            'id': 'id',
                            'type': 'type',
                            'state': 'state',
                            'vm_moref_id': 'vm_moref_id',
                            'ip_address': 'ip_address',
                            'hostname': 'hostname',
                            }

    def __init__(self,
                 id=None,
                 type=None,
                 state=None,
                 vm_moref_id=None,
                 ip_address=None,
                 hostname=None,
                ):
        """
        :type  id: :class:`str` or ``None``
        :param id: 
        :type  type: :class:`str` or ``None``
        :param type: Possible values are: 
            
            * :attr:`SiteRecoveryNode.TYPE_VRMS`
            * :attr:`SiteRecoveryNode.TYPE_SRM`
            * :attr:`SiteRecoveryNode.TYPE_VRS`
        :type  state: :class:`str` or ``None``
        :param state: Possible values are: 
            
            * :attr:`SiteRecoveryNode.STATE_DEPLOYING`
            * :attr:`SiteRecoveryNode.STATE_PROVISIONED`
            * :attr:`SiteRecoveryNode.STATE_READY`
            * :attr:`SiteRecoveryNode.STATE_DELETING`
            * :attr:`SiteRecoveryNode.STATE_FAILED`
            * :attr:`SiteRecoveryNode.STATE_CANCELED`
        :type  vm_moref_id: :class:`str` or ``None``
        :param vm_moref_id: 
        :type  ip_address: :class:`str` or ``None``
        :param ip_address: 
        :type  hostname: :class:`str` or ``None``
        :param hostname: 
        """
        self.id = id
        self.type = type
        self.state = state
        self.vm_moref_id = vm_moref_id
        self.ip_address = ip_address
        self.hostname = hostname
        VapiStruct.__init__(self)


SiteRecoveryNode._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.site_recovery_node', {
        'id': type.OptionalType(type.StringType()),
        'type': type.OptionalType(type.StringType()),
        'state': type.OptionalType(type.StringType()),
        'vm_moref_id': type.OptionalType(type.StringType()),
        'ip_address': type.OptionalType(type.StringType()),
        'hostname': type.OptionalType(type.StringType()),
    },
    SiteRecoveryNode,
    False,
    None))



class SiteRecoveryNodeVersion(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """
    NODE_TYPE_VRMS = "VRMS"
    """


    """
    NODE_TYPE_SRM = "SRM"
    """


    """
    NODE_TYPE_VRS = "VRS"
    """


    """



    _canonical_to_pep_names = {
                            'node_id': 'node_id',
                            'node_type': 'node_type',
                            'node_ip': 'node_ip',
                            'build_version': 'build_version',
                            'full_version': 'full_version',
                            }

    def __init__(self,
                 node_id=None,
                 node_type=None,
                 node_ip=None,
                 build_version=None,
                 full_version=None,
                ):
        """
        :type  node_id: :class:`str` or ``None``
        :param node_id: 
        :type  node_type: :class:`str` or ``None``
        :param node_type: Possible values are: 
            
            * :attr:`SiteRecoveryNodeVersion.NODE_TYPE_VRMS`
            * :attr:`SiteRecoveryNodeVersion.NODE_TYPE_SRM`
            * :attr:`SiteRecoveryNodeVersion.NODE_TYPE_VRS`
        :type  node_ip: :class:`str` or ``None``
        :param node_ip: 
        :type  build_version: :class:`BuildVersion` or ``None``
        :param build_version: 
        :type  full_version: :class:`str` or ``None``
        :param full_version: 
        """
        self.node_id = node_id
        self.node_type = node_type
        self.node_ip = node_ip
        self.build_version = build_version
        self.full_version = full_version
        VapiStruct.__init__(self)


SiteRecoveryNodeVersion._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.site_recovery_node_version', {
        'node_id': type.OptionalType(type.StringType()),
        'node_type': type.OptionalType(type.StringType()),
        'node_ip': type.OptionalType(type.StringType()),
        'build_version': type.OptionalType(type.ReferenceType(__name__, 'BuildVersion')),
        'full_version': type.OptionalType(type.StringType()),
    },
    SiteRecoveryNodeVersion,
    False,
    None))



class SiteRecoveryVersions(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'sddc_id': 'sddc_id',
                            'node_versions': 'node_versions',
                            'generated': 'generated',
                            }

    def __init__(self,
                 sddc_id=None,
                 node_versions=None,
                 generated=None,
                ):
        """
        :type  sddc_id: :class:`str` or ``None``
        :param sddc_id: 
        :type  node_versions: :class:`list` of :class:`SiteRecoveryNodeVersion` or ``None``
        :param node_versions: list of site recovery node version
        :type  generated: :class:`datetime.datetime` or ``None``
        :param generated: 
        """
        self.sddc_id = sddc_id
        self.node_versions = node_versions
        self.generated = generated
        VapiStruct.__init__(self)


SiteRecoveryVersions._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.site_recovery_versions', {
        'sddc_id': type.OptionalType(type.StringType()),
        'node_versions': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'SiteRecoveryNodeVersion'))),
        'generated': type.OptionalType(type.DateTimeType()),
    },
    SiteRecoveryVersions,
    False,
    None))



class SrmNode(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """
    TYPE_VRMS = "VRMS"
    """


    """
    TYPE_SRM = "SRM"
    """


    """
    TYPE_VRS = "VRS"
    """


    """
    STATE_DEPLOYING = "DEPLOYING"
    """


    """
    STATE_PROVISIONED = "PROVISIONED"
    """


    """
    STATE_READY = "READY"
    """


    """
    STATE_DELETING = "DELETING"
    """


    """
    STATE_FAILED = "FAILED"
    """


    """
    STATE_CANCELED = "CANCELED"
    """


    """



    _canonical_to_pep_names = {
                            'id': 'id',
                            'type': 'type',
                            'state': 'state',
                            'vm_moref_id': 'vm_moref_id',
                            'ip_address': 'ip_address',
                            'hostname': 'hostname',
                            'srm_extension_key_suffix': 'srm_extension_key_suffix',
                            'srm_extension_key': 'srm_extension_key',
                            }

    def __init__(self,
                 id=None,
                 type=None,
                 state=None,
                 vm_moref_id=None,
                 ip_address=None,
                 hostname=None,
                 srm_extension_key_suffix=None,
                 srm_extension_key=None,
                ):
        """
        :type  id: :class:`str` or ``None``
        :param id: 
        :type  type: :class:`str` or ``None``
        :param type: Possible values are: 
            
            * :attr:`SrmNode.TYPE_VRMS`
            * :attr:`SrmNode.TYPE_SRM`
            * :attr:`SrmNode.TYPE_VRS`
        :type  state: :class:`str` or ``None``
        :param state: Possible values are: 
            
            * :attr:`SrmNode.STATE_DEPLOYING`
            * :attr:`SrmNode.STATE_PROVISIONED`
            * :attr:`SrmNode.STATE_READY`
            * :attr:`SrmNode.STATE_DELETING`
            * :attr:`SrmNode.STATE_FAILED`
            * :attr:`SrmNode.STATE_CANCELED`
        :type  vm_moref_id: :class:`str` or ``None``
        :param vm_moref_id: 
        :type  ip_address: :class:`str` or ``None``
        :param ip_address: 
        :type  hostname: :class:`str` or ``None``
        :param hostname: 
        :type  srm_extension_key_suffix: :class:`str` or ``None``
        :param srm_extension_key_suffix: 
        :type  srm_extension_key: :class:`str` or ``None``
        :param srm_extension_key: 
        """
        self.id = id
        self.type = type
        self.state = state
        self.vm_moref_id = vm_moref_id
        self.ip_address = ip_address
        self.hostname = hostname
        self.srm_extension_key_suffix = srm_extension_key_suffix
        self.srm_extension_key = srm_extension_key
        VapiStruct.__init__(self)


SrmNode._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.srm_node', {
        'id': type.OptionalType(type.StringType()),
        'type': type.OptionalType(type.StringType()),
        'state': type.OptionalType(type.StringType()),
        'vm_moref_id': type.OptionalType(type.StringType()),
        'ip_address': type.OptionalType(type.StringType()),
        'hostname': type.OptionalType(type.StringType()),
        'srm_extension_key_suffix': type.OptionalType(type.StringType()),
        'srm_extension_key': type.OptionalType(type.StringType()),
    },
    SrmNode,
    False,
    None))



class Task(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """
    STATUS_STARTED = "STARTED"
    """


    """
    STATUS_CANCELING = "CANCELING"
    """


    """
    STATUS_FINISHED = "FINISHED"
    """


    """
    STATUS_FAILED = "FAILED"
    """


    """
    STATUS_CANCELED = "CANCELED"
    """


    """



    _canonical_to_pep_names = {
                            'id': 'id',
                            'version': 'version',
                            'created': 'created',
                            'user_id': 'user_id',
                            'user_name': 'user_name',
                            'updated': 'updated',
                            'updated_by_user_id': 'updated_by_user_id',
                            'updated_by_user_name': 'updated_by_user_name',
                            'end_time': 'end_time',
                            'error_message': 'error_message',
                            'tenant_id': 'tenant_id',
                            'params': 'params',
                            'resource_id': 'resource_id',
                            'resource_type': 'resource_type',
                            'parent_task_id': 'parent_task_id',
                            'retries': 'retries',
                            'start_time': 'start_time',
                            'status': 'status',
                            'sub_status': 'sub_status',
                            'task_type': 'task_type',
                            'task_version': 'task_version',
                            'task_progress_phases': 'task_progress_phases',
                            'progress_percent': 'progress_percent',
                            'estimated_remaining_minutes': 'estimated_remaining_minutes',
                            }

    def __init__(self,
                 id=None,
                 version=None,
                 created=None,
                 user_id=None,
                 user_name=None,
                 updated=None,
                 updated_by_user_id=None,
                 updated_by_user_name=None,
                 end_time=None,
                 error_message=None,
                 tenant_id=None,
                 params=None,
                 resource_id=None,
                 resource_type=None,
                 parent_task_id=None,
                 retries=None,
                 start_time=None,
                 status=None,
                 sub_status=None,
                 task_type=None,
                 task_version=None,
                 task_progress_phases=None,
                 progress_percent=None,
                 estimated_remaining_minutes=None,
                ):
        """
        :type  id: :class:`str`
        :param id: 
        :type  version: :class:`long`
        :param version: Version of this entity format: int32
        :type  created: :class:`datetime.datetime`
        :param created: 
        :type  user_id: :class:`str`
        :param user_id: User id that last updated this record
        :type  user_name: :class:`str`
        :param user_name: User name that last updated this record
        :type  updated: :class:`datetime.datetime`
        :param updated: 
        :type  updated_by_user_id: :class:`str`
        :param updated_by_user_id: User id that last updated this record
        :type  updated_by_user_name: :class:`str`
        :param updated_by_user_name: User name that last updated this record
        :type  end_time: :class:`datetime.datetime` or ``None``
        :param end_time: 
        :type  error_message: :class:`str` or ``None``
        :param error_message: 
        :type  tenant_id: :class:`str` or ``None``
        :param tenant_id: 
        :type  params: :class:`vmware.vapi.struct.VapiStruct` or ``None``
        :param params: 
        :type  resource_id: :class:`str` or ``None``
        :param resource_id: UUID of resources task is acting upon
        :type  resource_type: :class:`str` or ``None``
        :param resource_type: Type of resource being acted upon
        :type  parent_task_id: :class:`str` or ``None``
        :param parent_task_id: 
        :type  retries: :class:`long` or ``None``
        :param retries: 
        :type  start_time: :class:`datetime.datetime` or ``None``
        :param start_time: 
        :type  status: :class:`str` or ``None``
        :param status: Possible values are: 
            
            * :attr:`Task.STATUS_STARTED`
            * :attr:`Task.STATUS_CANCELING`
            * :attr:`Task.STATUS_FINISHED`
            * :attr:`Task.STATUS_FAILED`
            * :attr:`Task.STATUS_CANCELED`
        :type  sub_status: :class:`str` or ``None``
        :param sub_status: 
        :type  task_type: :class:`str` or ``None``
        :param task_type: 
        :type  task_version: :class:`str` or ``None``
        :param task_version: 
        :type  task_progress_phases: :class:`list` of :class:`TaskProgressPhase` or ``None``
        :param task_progress_phases: Task progress phases involved in current task execution
        :type  progress_percent: :class:`long` or ``None``
        :param progress_percent: Estimated progress percentage the task executed format: int32
        :type  estimated_remaining_minutes: :class:`long` or ``None``
        :param estimated_remaining_minutes: Estimated remaining time in minute of the task execution, < 0 means
            no estimation for the task. format: int32
        """
        self.id = id
        self.version = version
        self.created = created
        self.user_id = user_id
        self.user_name = user_name
        self.updated = updated
        self.updated_by_user_id = updated_by_user_id
        self.updated_by_user_name = updated_by_user_name
        self.end_time = end_time
        self.error_message = error_message
        self.tenant_id = tenant_id
        self.params = params
        self.resource_id = resource_id
        self.resource_type = resource_type
        self.parent_task_id = parent_task_id
        self.retries = retries
        self.start_time = start_time
        self.status = status
        self.sub_status = sub_status
        self.task_type = task_type
        self.task_version = task_version
        self.task_progress_phases = task_progress_phases
        self.progress_percent = progress_percent
        self.estimated_remaining_minutes = estimated_remaining_minutes
        VapiStruct.__init__(self)


Task._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.task', {
        'id': type.StringType(),
        'version': type.IntegerType(),
        'created': type.DateTimeType(),
        'user_id': type.StringType(),
        'user_name': type.StringType(),
        'updated': type.DateTimeType(),
        'updated_by_user_id': type.StringType(),
        'updated_by_user_name': type.StringType(),
        'end_time': type.OptionalType(type.DateTimeType()),
        'error_message': type.OptionalType(type.StringType()),
        'tenant_id': type.OptionalType(type.StringType()),
        'params': type.OptionalType(type.DynamicStructType('vmware.vapi.dynamic_struct', {}, VapiStruct)),
        'resource_id': type.OptionalType(type.StringType()),
        'resource_type': type.OptionalType(type.StringType()),
        'parent_task_id': type.OptionalType(type.StringType()),
        'retries': type.OptionalType(type.IntegerType()),
        'start_time': type.OptionalType(type.DateTimeType()),
        'status': type.OptionalType(type.StringType()),
        'sub_status': type.OptionalType(type.StringType()),
        'task_type': type.OptionalType(type.StringType()),
        'task_version': type.OptionalType(type.StringType()),
        'task_progress_phases': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'TaskProgressPhase'))),
        'progress_percent': type.OptionalType(type.IntegerType()),
        'estimated_remaining_minutes': type.OptionalType(type.IntegerType()),
    },
    Task,
    False,
    None))



class TaskProgressPhase(VapiStruct):
    """
    A task progress can be (but does NOT have to be) divided to more meaningful
    progress phases.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'id': 'id',
                            'name': 'name',
                            'progress_percent': 'progress_percent',
                            }

    def __init__(self,
                 id=None,
                 name=None,
                 progress_percent=None,
                ):
        """
        :type  id: :class:`str`
        :param id: The identifier of the task progress phase
        :type  name: :class:`str`
        :param name: The display name of the task progress phase
        :type  progress_percent: :class:`long`
        :param progress_percent: The percentage of the phase that has completed format: int32
        """
        self.id = id
        self.name = name
        self.progress_percent = progress_percent
        VapiStruct.__init__(self)


TaskProgressPhase._set_binding_type(type.StructType(
    'com.vmware.vmc.draas.model.task_progress_phase', {
        'id': type.StringType(),
        'name': type.StringType(),
        'progress_percent': type.IntegerType(),
    },
    TaskProgressPhase,
    False,
    None))




class StubFactory(StubFactoryBase):
    _attrs = {
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vmc.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Locale(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.vmc.locale'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _LocaleStub)
        self._VAPI_OPERATION_IDS = {}


    def set(self,
            vmc_locale,
            ):
        """
        Sets the locale for the session which is used for translating
        responses.

        :type  vmc_locale: :class:`com.vmware.vmc.model_client.VmcLocale`
        :param vmc_locale: The locale to be set. (required)
        :rtype: :class:`com.vmware.vmc.model_client.VmcLocale`
        :return: com.vmware.vmc.model.VmcLocale
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            Unauthorized
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        """
        return self._invoke('set',
                            {
                            'vmc_locale': vmc_locale,
                            })
class Orgs(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.vmc.orgs'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _OrgsStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            org,
            ):
        """
        Get details of organization

        :type  org: :class:`str`
        :param org: Organization identifier (required)
        :rtype: :class:`com.vmware.vmc.model_client.Organization`
        :return: com.vmware.vmc.model.Organization
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            Unauthorized
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Organization doesn't exist
        """
        return self._invoke('get',
                            {
                            'org': org,
                            })

    def list(self):
        """
        Return a list of all organizations the calling user (based on
        credential) is authorized on.


        :rtype: :class:`list` of :class:`com.vmware.vmc.model_client.Organization`
        :return: 
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            Unauthorized
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        """
        return self._invoke('list', None)
class _LocaleStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for set operation
        set_input_type = type.StructType('operation-input', {
            'vmc_locale': type.ReferenceType('com.vmware.vmc.model_client', 'VmcLocale'),
        })
        set_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),

        }
        set_input_value_validator_list = [
        ]
        set_output_validator_list = [
        ]
        set_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vmc/api/locale',
            request_body_parameter='vmc_locale',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        operations = {
            'set': {
                'input_type': set_input_type,
                'output_type': type.ReferenceType('com.vmware.vmc.model_client', 'VmcLocale'),
                'errors': set_error_dict,
                'input_value_validator_list': set_input_value_validator_list,
                'output_validator_list': set_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'set': set_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vmc.locale',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _OrgsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'org': type.StringType(),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vmc/api/orgs/{org}',
            path_variables={
                'org': 'org',
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for list operation
        list_input_type = type.StructType('operation-input', {})
        list_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vmc/api/orgs',
            path_variables={
            },
             header_parameters={
             },
            query_parameters={
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.vmc.model_client', 'Organization'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'list': {
                'input_type': list_input_type,
                'output_type': type.ListType(type.ReferenceType('com.vmware.vmc.model_client', 'Organization')),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vmc.orgs',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Locale': Locale,
        'Orgs': Orgs,
        'customer_support': 'com.vmware.vmc.customer_support_client.StubFactory',
        'model': 'com.vmware.vmc.model_client.StubFactory',
        'orgs': 'com.vmware.vmc.orgs_client.StubFactory',
    }




"""
This module is YATP (Yet another thread pool)
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015, 2017-2019 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

import six
import sys
import time
import threading

from six.moves import queue
from vmware.vapi.common.context import clear_event, get_event
from vmware.vapi.lib.log import get_vapi_logger

logger = get_vapi_logger(__name__)
MAX_TIMEOUT = 86400  # 1 day (24*60*60 secs)


class WorkItem(object):
    """ Work item """

    def __init__(self, fn, *args, **kwargs):
        """
        Work item constructor

        :type  fn: function
        :param fn: Work item functor
        :type  args: :class:`tuple`
        :param args: Work item functor positional parameters
        :type  kwargs: :class:`dict`
        :param kwargs: Work item functor key-value parameters
        """
        self.fn = fn
        self.args = args
        self.kwargs = kwargs
        self.ret = None
        self.err = None
        self.event = threading.Event()

    def join(self, timeout=MAX_TIMEOUT):
        """
        Wait for work item is done

        :type  timeout: :class:`float`
        :param timeout: Timeout in seconds
        :rtype: :class:`object`
        :return: Work item functor's return value
        """
        # Wait until work is done
        self.event.wait(timeout)

        # Throw exception if error occured
        if self.err:
            raise self.err  # pylint: disable=E0702
        return self.ret

    def done(self):
        """ Signal work item is done """
        self.event.set()

    def __enter__(self):
        """
        with statement enter

        :rtype:  :class:`WorkItem`
        :return: Work item object
        """
        return self

    def __exit__(self, typ, value, traceback):
        """ with statement exit """
        self.join()
        del self.event, self.fn, self.args, self.kwargs

## Thread pool
#


class ThreadPool(object):
    """ Thread pool """

    def __init__(self, min_workers=1, max_workers=8,
                 idle_timeout=5 * 60, thread_logger=None):
        """
        Thread pool constructor

        :type   min_workers: :class:`int`
        :params min_workers: Min number of worker threads
        :type   max_workers: :class:`int`
        :params max_workers: Max number of worker threads
        :type   idle_timeout: :class:`int`
        :params idle_timeout: Worker threads idle timeout
        :type   thread_logger: :class:`logging.logger`
        :params thread_logger: logger to use. Default to stdout
        """
        assert(min_workers >= 0)
        assert(min_workers <= max_workers)
        self.min_workers = min_workers
        self.max_workers = max_workers
        self.idle_timeout = idle_timeout
        self.workers = {}
        self.work_items = queue.Queue(0)
        self.lock = threading.Lock()
        self.shutting_down = False
        self.logger = thread_logger or logger
        for _ in range(0, self.min_workers):
            self._add_worker()

    def _log(self, msg):
        """ Log message """
        if self.logger:
            self.logger.info(msg)
        else:
            print(msg)

    def _worker(self):
        """ Thread pool worker """
        thd_name = threading.currentThread().getName()
        while not self.shutting_down:
            try:
                # Wait for request
                try:
                    work_item = self.work_items.get(timeout=self.idle_timeout)
                except queue.Empty:
                    work_item = None

                if not work_item:
                    # Worker idle timeout. Retire thread if needed
                    self.lock.acquire()
                    try:
                        done_thread = len(self.workers) > self.min_workers
                        if done_thread:
                            self._remove_worker(thd_name)
                            break
                        else:
                            continue
                    finally:
                        self.lock.release()
                elif self.shutting_down:
                    # Put the work item back to queue
                    self.work_items.put(work_item)
                    break

                # Start work
                work_item.ret = work_item.fn(
                    *work_item.args, **work_item.kwargs)
            except:  # pylint: disable=W0702
                if sys:
                    import traceback
                    errtype, errvalue, trace = sys.exc_info()
                    stack_trace = ' '.join(traceback.format_exception(
                        errtype, errvalue, trace))
                    self._log('%s caught exception: %s'
                              % (thd_name, str(errtype)))
                    self._log(stack_trace)
                    if work_item:
                        work_item.err = errvalue

                    # In case of exception from provider also set the event
                    # so we can throw the error rightaway to client rather
                    # than waiting for the timeout to expire.
                    accept_event = get_event()
                    if accept_event is not None:
                        accept_event.set()
                        clear_event()
                    #
                    # NOTE: See the Python documentation for sys.exc_info for a
                    # warning about an inefficiency in garbage collection and
                    # the need to delete the local variable to which stacktrace
                    # is assigned
                    try:
                        del trace
                    except:  # pylint: disable=W0702
                        pass
                else:
                    # System is dying and likely to be in undefined state.
                    # sys (and other imported modules) could be unloaded and set
                    # to None when we get here. Must quit as quickly as possible
                    return

            # Signal done on work_item
            work_item.done()

        # One less worker
        self.lock.acquire()
        try:
            self._remove_worker(thd_name)
        finally:
            self.lock.release()

    def _remove_worker(self, thd_name):
        """
        Remove a worker. Assume locked

        :type  thd_name: :class:`str`
        :param thd_name: Remove a worker thread with the thread name
        """
        self.workers.pop(thd_name, None)

    def _add_worker(self):
        """ Add a worker. Assume locked """
        if len(self.workers) < self.max_workers:
            thd = threading.Thread(target=self._worker)
            thd.setDaemon(True)
            thd.start()
            self.workers[thd.getName()] = thd

    def queue_work(self, fn, *args, **kwargs):
        """
        Queue work

        Returns a work_item when work is queued to work queue
        The work will start when a ready worker is available to process the work
        User could call {work_item}.join() to wait for the work item to finish

        :type  fn: function
        :param fn: Work item functor
        :type  args: :class:`tuple`
        :param args: Work item functor positional parameters
        :type  kwargs: :class:`dict`
        :param kwargs: Work item functor key-value parameters
        :rtype: :class:`WorkItem`
        :return: work item when work is queued to work queue
        """
        if self.shutting_down:
            return None

        # Add worker if needed
        self.lock.acquire()
        try:
            self._add_worker()
        finally:
            self.lock.release()

        work_item = WorkItem(fn, *args, **kwargs)
        self.work_items.put(work_item)
        return work_item

    @staticmethod
    def normalize_works(works):
        """
        Generator to return work in normalize form: (fn, args, kwargs)

        :type  works: iteratable of fn / (fn, args) / (fn, args, kwargs)
        :param works: An iteratable of possible functor form
        :rtype: :class:`tuple` of (fn, args, kwargs)
        :return: A normalize tuple of (functor, args, kwargs)
        """
        for work in works:
            args = ()
            kwargs = {}
            if six.callable(work):
                fn = work
            elif len(work) >= 3:
                fn, args, kwargs = work
            elif len(work) == 2:
                fn, args = work
            else:
                fn = work[0]
            yield (fn, args, kwargs)

    def queue_works_and_wait(self, works):
        """
        Queue a brunch of works and wait until all works are completed / error
        out

        :type  works: iteratable of fn / (fn, args) / (fn, args, kwargs)
        :param works: An iteratable of possible functor form
        :rtype:  :class:`list` of {tuple} of :class:`bool`,:class:`object`
                 or :class:`bool`,:class:`Exception`
        :return: A list of (True, return val) / (False, exception) when all
            works done
        """
        work_items = [self.queue_work(fn, *args, **kwargs)
                      for fn, args, kwargs in self.normalize_works(works)]
        results = []
        for work in work_items:
            if work:
                try:
                    time.sleep(0.1)
                    ret = work.join()
                    results.append((True, ret))
                except:  # pylint: disable=W0702
                    results.append((False, sys.exc_info()[1]))
            else:
                # No work queued
                results.append((False, None))

        return results

    def queue_work_and_wait(self, fn, *args, **kwargs):
        """
        Queue a work and wait until the work is completed / error out

        :type  fn: function
        :param fn: Work item functor
        :type  args: :class:`tuple`
        :param args: Work item functor positional parameters
        :type  kwargs: :class:`dict`
        :param kwargs: Work item functor key-value parameters
        :rtype: :class:`tuple` of (:class:`bool`, :class:`object`) or
                (:class:`bool`, :class:`Exception`)
        :return: (True, return val) / (False, exception) when work is done
        """
        return self.queue_works_and_wait([(fn, args, kwargs)])[0]

    def shutdown(self, no_wait=False):
        """
        Shuthdown this thread pool

        :type  no_wait: :class:`bool`
        :param no_wait: Set to True to return immediately without waiting for
            all workers to quit
        """
        # Set myself as shutting down.
        if self.shutting_down:
            return
        self.shutting_down = True

        # Queue a fake work item
        work_item = object()
        self.work_items.put(work_item)

        # Wait until all workers quit
        if not no_wait:
            self._log("shutdown: Waiting for workers to quit...")
            while True:
                self.lock.acquire()
                try:
                    num_workers = len(self.workers)
                finally:
                    self.lock.release()

                # done if no worker left or not making progress
                if num_workers == 0:
                    break

                time.sleep(0.1)
            self._log("shutdown: All workers quit")

    def __del__(self):
        """ Destructor """
        self.shutdown()

    def __enter__(self):
        """ with statment enter """
        return self

    def __exit__(self, typ, value, traceback):
        """ with statment exit """
        self.shutdown()


# Thread pool instance
_thread_pool = None


def get_threadpool(max_workers=3):
    """
    Get thread pool instance.

    :type  max_workers: :class:`int`
    :param max_workers: Max number of worker threads
    """
    global _thread_pool
    if max_workers > 0 and _thread_pool is None:
        logger.info('Creating thread pool instance with %s threads',
                    max_workers)
        _thread_pool = ThreadPool(max_workers=max_workers)

    return _thread_pool



"""
Authentication Handler interface
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long


class AuthenticationHandler(object):
    """
    The AuthenticationHandler interface is used to verify the authentication
    data provided in the security context against an identity source.
    """

    def authenticate(self, ctx):
        """
        Verifies the provided authentication data against the relevant identity
        source.

        :type  ctx: :class:`vmware.vapi.core.SecurityContext`
        :param ctx: Security context for the method

        :rtype: :class:`vmware.vapi.security.user_identity.UserIdentity`
        :return: Authentication Identity for successful authentication,
                 False for failed authentication and None for invalid handler.
        """
        raise NotImplementedError

    def get_supported_scheme(self):
        """
        Get the scheme supported by this handler
        """
        raise NotImplementedError

    def __hash__(self):
        return str(self).__hash__()



"""
Representation of an IDL type for the use of the Python language bindings.
"""
__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

import six

from vmware.vapi.data.definition import (
    IntegerDefinition, DoubleDefinition, BooleanDefinition, StringDefinition,
    BlobDefinition, VoidDefinition, OpaqueDefinition, ListDefinition,
    StructDefinition, OptionalDefinition, SecretDefinition, StructRefDefinition,
    ErrorDefinition, ReferenceResolver, DynamicStructDefinition,
    AnyErrorDefinition)
from vmware.vapi.lib.constants import MAP_ENTRY
from vmware.vapi.lib.load import dynamic_import
from vmware.vapi.lib.visitor import VapiVisitor

MAP_KEY_FIELD = 'key'
MAP_VALUE_FIELD = 'value'


class BindingType(object):
    """
    Representation of an IDL type for the use of the Python language bindings

    :type  definition: :class:`vmware.vapi.data.definition.DataDefinition`
    :param definition: Data definition corresponding to this binding type
    """
    def __init__(self):
        self._definition = None

    def accept(self, visitor):
        """
        Applies a visitor to this BindingType

        :type  visitor: :class:`BindingTypeVisitor`
        :param visitor: visitor operating on the BindingType
        """
        visitor.visit(self)

    @property
    def definition(self):
        """
        Generate the data defintion corresponding to this binding type
        """
        # This will make sure that we only generate the definition once
        # for every instance of binding type
        if self._definition is None:
            self._definition = TypeUtil.convert_to_data_definition(self)
        return self._definition


class VoidType(BindingType):
    """
    Representation of void IDL type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class IntegerType(BindingType):
    """
    Representation of integer IDL Type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class DoubleType(BindingType):
    """
    Representation of float IDL Type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class StringType(BindingType):
    """
    Representation of string IDL Type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class SecretType(BindingType):
    """
    Representation of @secret IDL annotation in Python Binding. @secret
    annotation can only be applied to strings.
    """
    def __init__(self):
        BindingType.__init__(self)


class BooleanType(BindingType):
    """
    Representation of boolean IDL Type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class BlobType(BindingType):
    """
    Representation of binary IDL Type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class OptionalType(BindingType):
    """
    Representation of optional IDL annotation in Python Binding

    :type element_type: :class:`BindingType`
    :ivar element_type: element type
    """
    def __init__(self, element_type):
        if not isinstance(element_type, BindingType):
            raise TypeError(
                'Expected an instance of BindingType as element_type parameter '
                + 'of OptionalType() but got %s' % type(element_type).__name__)
        self._element_type = element_type
        BindingType.__init__(self)

    @property
    def element_type(self):
        """
        Return the element type of this ListType

        :rtype: :class:`BindingType`
        :return: element type
        """
        return self._element_type


class ListType(BindingType):
    """
    Representation of List IDL type in Python Binding

    :type element_type: :class:`BindingType`
    :ivar element_type: element type
    """
    def __init__(self, element_type):
        if not isinstance(element_type, BindingType):
            raise TypeError(
                'Expected an instance of BindingType as element_type parameter '
                + 'of ListType() but got %s' % type(element_type).__name__)
        self._element_type = element_type
        BindingType.__init__(self)

    @property
    def element_type(self):
        """
        Return the element type of this ListType

        :rtype: :class:`BindingType`
        :return: element type
        """
        return self._element_type


class SetType(BindingType):
    """
    Representation of Set IDL type in Python Binding

    :type element_type: :class:`BindingType`
    :ivar element_type: element type
    """
    def __init__(self, element_type):
        if not isinstance(element_type, BindingType):
            raise TypeError(
                'Expected an instance of BindingType as element_type parameter '
                + 'of SetType() but got %s' % type(element_type).__name__)
        self._element_type = element_type
        BindingType.__init__(self)

    @property
    def element_type(self):
        """
        Return the element type of this SetType

        :rtype: :class:`BindingType`
        :return: element type
        """
        return self._element_type


class MapType(BindingType):
    """
    Representation of Map IDL type in Python Binding

    :type key_type: :class:`BindingType`
    :ivar key_type: map key type
    :type value_type: :class:`BindingType`
    :ivar value_type: map value type
    """
    def __init__(self, key_type, value_type):
        if (not isinstance(key_type, BindingType)
                or not isinstance(value_type, BindingType)):
            raise TypeError(
                'Expected an instance of BindingType for both key_type and '
                'value_type parameters of MapType() but got '
                '%s and %s respectively'
                % (type(key_type).__name__, type(value_type).__name__))
        self._key_type = key_type
        self._value_type = value_type
        BindingType.__init__(self)

    @property
    def key_type(self):
        """
        Return the key type of this MapType

        :rtype: :class:`BindingType`
        :return: key type
        """
        return self._key_type

    @property
    def value_type(self):
        """
        Return the value type of this MapType

        :rtype: :class:`BindingType`
        :return: value type
        """
        return self._value_type


class StructType(BindingType):
    """
    Representation of Structure IDL type in Python Binding

    :type name: :class:`str`
    :ivar name: Name of the structure
    :type binding_class: :class:`vmware.vapi.bindings.struct.VapiStruct`
    :ivar binding_class: Reference to the Python native class corresponding
                         to this structure
    :type is_model: :class:`bool`
    :ivar is_model: True if the structure is marked as Model, False otherwise
    :type model_keys: :class:`list` of :class:`str` or :class:`None`
    :ivar model_keys: List of model keys for the structure if it is marked as
        Model
    """
    def __init__(self, name, fields, binding_class=None, is_model=False,
                 model_keys=None):
        if not isinstance(name, six.string_types):
            raise TypeError('Structure name must be a string')
        if not isinstance(fields, dict):
            raise TypeError('Structure fields must be a dict')
        for field_name, binding_type in six.iteritems(fields):
            if not isinstance(binding_type, BindingType):
                raise TypeError(
                    'Type of the field %s in StructType should be a BindingType'
                    ' but got %s for one of the fields'
                    % (field_name, type(binding_type).__name__))
        self._name = name
        self._binding_class = binding_class
        self._field_map = fields
        self._is_model = is_model
        self._model_keys = model_keys
        BindingType.__init__(self)

    @property
    def name(self):
        """
        Returns the name of the StructType

        :rtype: :class:`str`
        :return: Name of the StructType
        """
        return self._name

    @property
    def binding_class(self):
        """
        Returns the reference to the Python native class
        corresponding to this structure

        :rtype: :class:`vmware.vapi.bindings.struct.VapiStruct`
        :return: Reference to the python native class
        """
        return self._binding_class

    @property
    def is_model(self):
        """
        Check if the Struct is marked as model

        :rtype: :class:`bool`
        :return: True if the Struct is marked as model, False otherwise
        """
        return self._is_model

    @property
    def model_keys(self):
        """
        Returns list of model keys for the Struct if it is marked as model

        :rtype: :class:`list` of :class:`str` or None
        :return: List of model keys for the Struct if it is marked as model
        """
        return self._model_keys

    def get_field_names(self):
        """
        Returns the list of field names present in this StructType

        :rtype: :class:`list` of :class:`str`
        :return: List of field names
        """
        return list(self._field_map.keys())

    def get_field(self, field_name):
        """
        Returns the BindingType of the argument

        :type  field_name: :class:`str`
        :param field_name: Field name
        :rtype: :class:`BindingType`
        :return: BindingType of the field specified
        """
        return self._field_map.get(field_name)


class ErrorType(StructType):
    """
    Representation of Error IDL type in Python Binding

    :type definition: :class:`vmware.vapi.data.ErrorDefinition`
    :ivar definition: type representation in the API runtime
    :type name: :class:`str`
    :ivar name: Name of the structure
    :type binding_class: :class:`vmware.vapi.bindings.error.VapiError`
    :ivar binding_class: Reference to the Python native class corresponding
                         to this error
    """
    def __init__(self, name, fields, binding_class=None):
        StructType.__init__(self, name, fields, binding_class)


class ReferenceType(BindingType):
    """
    Reference type to resolve references lazily.

    :type resolved_type: :class:`StructType` or :class:`EnumType`
    :ivar resolved_type: Resolved reference type
    """
    def __init__(self, context_name, type_name):
        """
        Initialize ReferenceType

        :type  context_name: :class:`str`
        :param context_name: Name of the module that has the type
        :type  type_name: :class:`str`
        :param type_name: Fully qualified name of the type reference. i.e.
            if the type Bar is nested inside type Foo, it would be Foo.Bar
        """
        if not isinstance(type_name, six.string_types):
            raise TypeError('Type name should be a string')
        self._type_name = type_name
        if not isinstance(context_name, six.string_types):
            raise TypeError('Context name should be a string')
        self._context_name = context_name
        self._resolved_type = None
        BindingType.__init__(self)

    def _resolve(self):
        """
        Resolves the struct or enum reference
        """
        tokens = self._type_name.split('.')
        target = dynamic_import(self._context_name)
        # Get the required reference by walking down the fully qualified name
        for token in tokens:
            target = getattr(target, token)
        self._resolved_type = target.get_binding_type()

    @property
    def resolved_type(self):
        """
        Returns the resolved struct type or enum type

        :rtype: :class:`StructType` or
            :class:`EnumType`
        :return: Resolved struct type or enum type
        """
        if self._resolved_type is None:
            self._resolve()
        return self._resolved_type


class OpaqueType(BindingType):
    """
    Representation of Opaque IDL annotation in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class DynamicStructType(StructType):
    """
    Representation of StructValue IDL annotation in Python Binding

    :type has_fields_of_type: :class:`ReferenceType`
    :ivar has_fields_of_type: List of reference types whose fields need to be
        present in the StructValue for this DynamicStruct type
    """
    def __init__(self, name, fields, binding_class=None,
                 has_fields_of_type=None):
        """
        Initialize DynamicStructType

        :type  name: :class:`str`
        :param name: Name of the Structure
        :type  fields: :class:`dict` of :class:`str` and :class:`BindingType`
        :param fields: Map of field name and field binding type
        :type  binding_class:
            :class:`vmware.vapi.data.definition.DataDefinition`
        :param binding_class: Data definition for this type
        :type  has_fields_of_type: :class:`ReferenceType`
        :param has_fields_of_type: List of reference types whose fields need to
            be present in the StructValue for this DynamicStruct type
        """
        StructType.__init__(self, name, fields, binding_class)
        self._has_fields_of_type = has_fields_of_type

    @property
    def has_fields_of_type(self):
        """
        Returns the has_fields_of_type

        :rtype  :class:`ReferenceType`
        :return List of reference types whose fields need to be present in the
            StructValue for this DynamicStruct type
        """
        return self._has_fields_of_type


class AnyErrorType(BindingType):
    """
    Representation of Exception type in Python Binding
    """
    def __init__(self):
        """
        Initialize AnyErrorType
        """
        BindingType.__init__(self)


class DateTimeType(BindingType):
    """
    Representation of datetime IDL Type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class URIType(BindingType):
    """
    Representation of URI IDL Type in Python Binding
    """
    def __init__(self):
        BindingType.__init__(self)


class EnumType(BindingType):
    """
    Representation of enum IDL Type in Python Binding

    :type name: :class:`str`
    :ivar name: Name of the enum
    :type binding_class: :class:`vmware.vapi.bindings.struct.VapiStruct`
    :ivar binding_class: Reference to the Python native class corresponding
                         to this structure
    """
    def __init__(self, name, binding_class):
        BindingType.__init__(self)
        self._name = name
        self._binding_class = binding_class

    @property
    def name(self):
        """
        Returns the name of the EnumType

        :rtype: :class:`str`
        :return: Name of the EnumType
        """
        return self._name

    @property
    def binding_class(self):
        """
        Returns the reference to the Python native class
        corresponding to this structure
        """
        return self._binding_class


class IdType(BindingType):
    """
    Representation of ID IDL type in Python Binding

    :type resolved_types: :class:`list` of :class:`str` or :class:`str` or
        :class:`None`
    :ivar resolved_types: Resource type(s) for the ID
    :type resource_type_field_name: :class:`str` or :class:`None`
    :ivar resource_type_field_name: Name of the field specifying the resource
        type
    """
    def __init__(self, resource_types=None, resource_type_field_name=None):
        BindingType.__init__(self)
        self._resource_types = resource_types
        self._resource_type_field_name = resource_type_field_name

    @property
    def resource_types(self):
        """
        Returns the Resource type(s) for the ID field

        :rtype: :class:`list` of :class:`str` or :class:`str` or :class:`None`
        :return: Resource type(s) for the ID
        """
        return self._resource_types

    @property
    def resource_type_field_name(self):
        """
        Returns the name of the field specifying the resource type

        :rtype: :class:`str`
        :return: Name of the field specifying the resource type
        """
        return self._resource_type_field_name


class BindingTypeVisitor(VapiVisitor):
    """
    Base no-op implementation of a BindingType visitor
    """
    def __init__(self):
        """
        Initialize BindingTypeVisitor
        """
        VapiVisitor.__init__(self, 'Type')

    def visit_void(self, typ):
        """
        Visit a void value (i.e. None)

        :type  typ: :class:`VoidType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_integer(self, typ):
        """
        Visit an integer value

        :type  typ: :class:`IntegerType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_double(self, typ):
        """
        Visit a double value

        :type  typ: :class:`DoubleType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_string(self, typ):
        """
        Visit a string value

        :type  typ: :class:`StringType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_boolean(self, typ):
        """
        Visit a boolean value

        :type  typ: :class:`BooleanType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_blob(self, typ):
        """
        Visit a blob value

        :type  typ: :class:`BlobType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_optional(self, typ):
        """
        Visit an optional value

        :type  typ: :class:`OptionalType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_list(self, typ):
        """
        Visit a list value

        :type  typ: :class:`ListType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_struct(self, typ):
        """
        Visit a struct value

        :type  typ: :class:`StructType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_dynamic_struct(self, typ):
        """
        Visit a struct value

        :type  typ: :class:`DynamicStructType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_any_error(self, typ):
        """
        Visit an error value

        :type  typ: :class:`AnyErrorType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_opaque(self, typ):
        """
        Visit an opaque value.

        :type  typ: :class:`OpaqueType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_secret(self, typ):
        """
        Visit a secret value

        :type  typ: :class:`SecretType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_date_time(self, typ):
        """
        Visit a datetime value

        :type  typ: :class:`DateTimeType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_uri(self, typ):
        """
        Visit an URI value

        :type  typ: :class:`URIType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_enum(self, typ):
        """
        Visit a enum value

        :type  typ: :class:`EnumType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_error(self, typ):
        """
        Visit an error type

        :type  typ: :class:`ErrorType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_reference(self, typ):
        """
        Visit a reference type

        :type  typ: :class:`ReferenceType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError

    def visit_id(self, typ):
        """
        Visit a ID value

        :type  typ: :class:`IdType`
        :param typ: Binding type of the value
        """
        raise NotImplementedError


class DataDefinitionBuilder(BindingTypeVisitor):
    """
    Builds DataDefinition by visiting a BindingType
    """
    def __init__(self, ctx, seen_structures):
        """
        Initialize DataDefinitionBuilder

        :type  ctx: :class:`vmware.vapi.data.definition.ReferenceResolver`
        :param ctx: Data definition reference resolver object
        :type  seen_structures: :class:`list` or :class:`str`
        :param seen_structures: List of structures seen
        """
        BindingTypeVisitor.__init__(self)
        self._ctx = ctx
        self._seen_structures = seen_structures
        self._out_value = None

    def get_out_value(self):
        """
        Returns the data definition

        :rtype: :class:`vmware.vapi.data.definition.DataDefinition`
        :return: Data definition
        """
        return self._out_value

    def visit_void(self, typ):
        """
        Visit a void value (i.e. None)

        :type  typ: :class:`VoidType`
        :param typ: Binding type of the value
        """
        self._out_value = VoidDefinition()

    def visit_integer(self, typ):
        """
        Visit an integer value

        :type  typ: :class:`IntegerType`
        :param typ: Binding type of the value
        """
        self._out_value = IntegerDefinition()

    def visit_double(self, typ):
        """
        Visit a double value

        :type  typ: :class:`DoubleType`
        :param typ: Binding type of the value
        """
        self._out_value = DoubleDefinition()

    def visit_string(self, typ):
        """
        Visit a string value

        :type  typ: :class:`StringType`
        :param typ: Binding type of the value
        """
        self._out_value = StringDefinition()

    def visit_boolean(self, typ):
        """
        Visit a boolean value

        :type  typ: :class:`BooleanType`
        :param typ: Binding type of the value
        """
        self._out_value = BooleanDefinition()

    def visit_blob(self, typ):
        """
        Visit a blob value

        :type  typ: :class:`BlobType`
        :param typ: Binding type of the value
        """
        self._out_value = BlobDefinition()

    def visit_optional(self, typ):
        """
        Visit an optional value

        :type  typ: :class:`OptionalType`
        :param typ: Binding type of the value
        """
        typ.element_type.accept(self)
        element_def = self._out_value
        self._out_value = OptionalDefinition(element_def)

    def visit_list(self, typ):
        """
        Visit a list value

        :type  typ: :class:`ListType`
        :param typ: Binding type of the value
        """
        typ.element_type.accept(self)
        element_def = self._out_value
        self._out_value = ListDefinition(element_def)

    def visit_set(self, typ):
        """
        Visit a set value

        :type  typ: :class:`SetType`
        :param typ: Binding type of the value
        """
        typ.element_type.accept(self)
        element_def = self._out_value
        self._out_value = ListDefinition(element_def)

    def visit_map(self, typ):
        """
        Visit a map value

        :type  typ: :class:`MapType`
        :param typ: Binding type of the value
        """
        field_defs = []
        typ.key_type.accept(self)
        key_def = self._out_value
        field_defs.append((MAP_KEY_FIELD, key_def))
        typ.value_type.accept(self)
        value_def = self._out_value

        field_defs.append((MAP_VALUE_FIELD, value_def))
        element_def = StructDefinition(MAP_ENTRY, field_defs)
        self._out_value = ListDefinition(element_def)

    def visit_struct(self, typ):
        """
        Visit a struct value

        :type  typ: :class:`StructType`
        :param typ: Binding type of the value
        """
        if typ.name in self._seen_structures:
            if self._ctx.is_defined(typ.name):
                self._out_value = self._ctx.get_definition(typ.name)
            else:
                self._out_value = StructRefDefinition(typ.name)
                self._ctx.add_reference(self._out_value)
        else:
            self._seen_structures.append(typ.name)
            field_defs = []
            for field_name in typ.get_field_names():
                field_type = typ.get_field(field_name)
                field_type.accept(self)
                field_defs.append((field_name, self._out_value))
            self._out_value = StructDefinition(typ.name, field_defs)
            self._ctx.add_definition(self._out_value)

    def visit_dynamic_struct(self, typ):
        """
        Visit a struct value

        :type  typ: :class:`DynamicStructType`
        :param typ: Binding type of the value
        """
        self._out_value = DynamicStructDefinition()

    def visit_any_error(self, typ):
        """
        Visit an error value

        :type  typ: :class:`AnyErrorType`
        :param typ: Binding type of the value
        """
        self._out_value = AnyErrorDefinition()

    def visit_opaque(self, typ):
        """
        Visit an opaque value.

        :type  typ: :class:`OpaqueType`
        :param typ: Binding type of the value
        """
        self._out_value = OpaqueDefinition()

    def visit_secret(self, typ):
        """
        Visit a secret value

        :type  typ: :class:`SecretType`
        :param typ: Binding type of the value
        """
        self._out_value = SecretDefinition()

    def visit_date_time(self, typ):
        """
        Visit a datetime value

        :type  typ: :class:`DateTimeType`
        :param typ: Binding type of the value
        """
        self._out_value = StringDefinition()

    def visit_uri(self, typ):
        """
        Visit an URI value

        :type  typ: :class:`URIType`
        :param typ: Binding type of the value
        """
        self._out_value = StringDefinition()

    def visit_enum(self, typ):
        """
        Visit a enum value

        :type  typ: :class:`EnumType`
        :param typ: Binding type of the value
        """
        self._out_value = StringDefinition()

    def visit_reference(self, typ):
        """
        Visit a reference type

        :type  typ: :class:`ReferenceType`
        :param typ: Binding type of the value
        """
        typ.resolved_type.accept(self)

    def visit_error(self, typ):
        """
        Visit an error type

        :type  typ: :class:`ErrorType`
        :param typ: Binding type of the value
        """
        field_defs = []
        for field_name in typ.get_field_names():
            field_type = typ.get_field(field_name)
            field_type.accept(self)
            field_defs.append((field_name, self._out_value))
        self._out_value = ErrorDefinition(typ.name, field_defs)

    def visit_id(self, typ):
        """
        Visit a ID value

        :type  typ: :class:`IdType`
        :param typ: Binding type of the value
        """
        self._out_value = StringDefinition()


class TypeUtil(object):
    """
    Converts a BindingType object to DataDefinition object
    """

    @staticmethod
    def convert_to_data_definition(binding_type):
        """
        Converts a BindingType object to DataDefinition object

        :type  binding_type: :class:`BindingType`
        :param binding_type: Binding type
        :rtype: :class:`vmware.vapi.data.definition.DataDefinition`
        :return: DataDefinition
        """
        ctx = ReferenceResolver()
        seen_structures = []
        visitor = DataDefinitionBuilder(ctx, seen_structures)
        binding_type.accept(visitor)
        ctx.resolve()
        return visitor.get_out_value()



# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.namespace_management.networks.edges.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata

class LoadBalancerSize(Enum):
    """
    The ``LoadBalancerSize`` class enumerates load balancer sizes supported by
    NSX. This enumeration was added in vSphere API 8.0.0.1.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    SMALL = None
    """
    :attr:`LoadBalancerSize.SMALL` is a load balancer that hosts up to 20
    virtual servers. This class attribute was added in vSphere API 8.0.0.1.

    """
    MEDIUM = None
    """
    :attr:`LoadBalancerSize.MEDIUM` is a load balancer that hosts up to 100
    virtual servers. This class attribute was added in vSphere API 8.0.0.1.

    """
    LARGE = None
    """
    :attr:`LoadBalancerSize.LARGE` is a load balancer that hosts up to 1000
    virtual servers. This class attribute was added in vSphere API 8.0.0.1.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`LoadBalancerSize` instance.
        """
        Enum.__init__(string)

LoadBalancerSize._set_values({
    'SMALL': LoadBalancerSize('SMALL'),
    'MEDIUM': LoadBalancerSize('MEDIUM'),
    'LARGE': LoadBalancerSize('LARGE'),
})
LoadBalancerSize._set_binding_type(type.EnumType(
    'com.vmware.vcenter.namespace_management.networks.edges.load_balancer_size',
    LoadBalancerSize))



class NSXRoutingMode(Enum):
    """
    ``NSXRoutingMode`` enum defines an enumeration of available routing modes.
    This enumeration was added in vSphere API 8.0.0.1.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    ROUTED = None
    """
    :attr:`NSXRoutingMode.ROUTED` configures NSX-T to route directly to Pods
    cluster IP addresses. This class attribute was added in vSphere API
    8.0.0.1.

    """
    NAT = None
    """
    :attr:`NSXRoutingMode.NAT` uses network address translation and the
    :attr:`NSXConfig.egress_ip_ranges` to route traffic out of the cluster.
    This class attribute was added in vSphere API 8.0.0.1.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`NSXRoutingMode` instance.
        """
        Enum.__init__(string)

NSXRoutingMode._set_values({
    'ROUTED': NSXRoutingMode('ROUTED'),
    'NAT': NSXRoutingMode('NAT'),
})
NSXRoutingMode._set_binding_type(type.EnumType(
    'com.vmware.vcenter.namespace_management.networks.edges.NSX_routing_mode',
    NSXRoutingMode))



class EdgeProvider(Enum):
    """
    ``EdgeProvider`` describes the supported available edge services. An NSX
    edge are used on NSX-T networks, but "Advanced NSX" and HAProxy edges are
    used on VDS networks. This enumeration was added in vSphere API 8.0.0.1.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    HAPROXY = None
    """
    :attr:`EdgeProvider.HAPROXY` is an HAProxy load balancer fronted by the
    Data Plane API. This class attribute was added in vSphere API 8.0.0.1.

    """
    NSX = None
    """
    :attr:`EdgeProvider.NSX` specifies NSX-T managed edge services. This class
    attribute was added in vSphere API 8.0.0.1.

    """
    NSX_ADVANCED = None
    """
    :attr:`EdgeProvider.NSX_ADVANCED` specifies the NSX Advanced Load Balancer
    and Ingress. This class attribute was added in vSphere API 8.0.0.1.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`EdgeProvider` instance.
        """
        Enum.__init__(string)

EdgeProvider._set_values({
    'HAPROXY': EdgeProvider('HAPROXY'),
    'NSX': EdgeProvider('NSX'),
    'NSX_ADVANCED': EdgeProvider('NSX_ADVANCED'),
})
EdgeProvider._set_binding_type(type.EnumType(
    'com.vmware.vcenter.namespace_management.networks.edges.edge_provider',
    EdgeProvider))




class Edge(VapiStruct):
    """
    ``Edge`` class contains configuration for network traffic entering and
    exiting a Supervisor. This class was added in vSphere API 8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """

    _validator_list = [
        UnionValidator(
            'provider',
            {
                'HAPROXY' : [('haproxy', True)],
                'NSX' : [('nsx', True)],
                'NSX_ADVANCED' : [('nsx_advanced', True)],
            }
        ),
    ]



    def __init__(self,
                 id=None,
                 load_balancer_address_ranges=None,
                 haproxy=None,
                 nsx=None,
                 nsx_advanced=None,
                 provider=None,
                ):
        """
        :type  id: :class:`str` or ``None``
        :param id: :attr:`Edge.id` is a unique identifier that can be referenced for
            updates. This attribute was added in vSphere API 8.0.0.1.
            When clients pass a value of this class as a parameter, the
            attribute must be an identifier for the resource type:
            ``com.vmware.vcenter.namespace_management.network.edge.Edge``. When
            methods return a value of this class as a return value, the
            attribute will be an identifier for the resource type:
            ``com.vmware.vcenter.namespace_management.network.edge.Edge``.
            If unset, an ID will be automatically generated.
        :type  load_balancer_address_ranges: :class:`list` of :class:`com.vmware.vcenter.namespace_management.networks_client.IPRange` or ``None``
        :param load_balancer_address_ranges: :attr:`Edge.load_balancer_address_ranges` defines the list of
            addresses that a load balancer can consume to publish Kubernetes
            services. This range must contain at least one IP address unless
            the AVI provider is selected. It is recommended to supply a large
            enough range to support load balancers requested for pods in the
            control plane and Tanzu Clusters. This attribute was added in
            vSphere API 8.0.0.1.
            This field must be set if :attr:`Edge.provider` is not an NSX
            Advanced Load Balancer,
        :type  haproxy: :class:`HAProxyConfig`
        :param haproxy: :attr:`Edge.haproxy` defines configuration for the HAProxy Load
            Balancer. This attribute was added in vSphere API 8.0.0.1.
            This attribute is optional and it is only relevant when the value
            of ``provider`` is :attr:`EdgeProvider.HAPROXY`.
        :type  nsx: :class:`NSXConfig`
        :param nsx: :attr:`Edge.nsx` defines configuration for the NSX Load Balancer.
            This attribute was added in vSphere API 8.0.0.1.
            This attribute is optional and it is only relevant when the value
            of ``provider`` is :attr:`EdgeProvider.NSX`.
        :type  nsx_advanced: :class:`NSXAdvancedLBConfig`
        :param nsx_advanced: :attr:`Edge.nsx_advanced` defines configuration for the NSX
            Advanced Load Balancer and Ingress Software. This attribute was
            added in vSphere API 8.0.0.1.
            This attribute is optional and it is only relevant when the value
            of ``provider`` is :attr:`EdgeProvider.NSX_ADVANCED`.
        :type  provider: :class:`EdgeProvider` or ``None``
        :param provider: :attr:`Edge.provider` specifies the vendor providing edge services.
            This attribute was added in vSphere API 8.0.0.1.
            A provider is required to enable edge services for a cluster.
        """
        self.id = id
        self.load_balancer_address_ranges = load_balancer_address_ranges
        self.haproxy = haproxy
        self.nsx = nsx
        self.nsx_advanced = nsx_advanced
        self.provider = provider
        VapiStruct.__init__(self)


Edge._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.networks.edges.edge', {
        'id': type.OptionalType(type.IdType()),
        'load_balancer_address_ranges': type.OptionalType(type.ListType(type.ReferenceType('com.vmware.vcenter.namespace_management.networks_client', 'IPRange'))),
        'haproxy': type.OptionalType(type.ReferenceType(__name__, 'HAProxyConfig')),
        'nsx': type.OptionalType(type.ReferenceType(__name__, 'NSXConfig')),
        'nsx_advanced': type.OptionalType(type.ReferenceType(__name__, 'NSXAdvancedLBConfig')),
        'provider': type.OptionalType(type.ReferenceType(__name__, 'EdgeProvider')),
    },
    Edge,
    False,
    None))



class NSXAdvancedLBConfig(VapiStruct):
    """
    ``NSXAdvancedLBConfig`` class is used to describe the NSX Advanced Load
    Balancer configuration. This class was added in vSphere API 8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 server=None,
                 username=None,
                 password=None,
                 certificate_authority_chain=None,
                ):
        """
        :type  server: :class:`Server`
        :param server: :attr:`NSXAdvancedLBConfig.server` is the address for the Avi
            Controller, used to configure Virtual Servers. This attribute was
            added in vSphere API 8.0.0.1.
        :type  username: :class:`str`
        :param username: :attr:`NSXAdvancedLBConfig.username` is used by the AVI Kubernetes
            Operator to program the Avi Controller. This attribute was added in
            vSphere API 8.0.0.1.
        :type  password: :class:`str`
        :param password: :attr:`NSXAdvancedLBConfig.password` is the password for the
            username. This attribute was added in vSphere API 8.0.0.1.
        :type  certificate_authority_chain: :class:`str`
        :param certificate_authority_chain: :attr:`NSXAdvancedLBConfig.certificate_authority_chain` contains
            PEM-encoded CA chain which is used to verify x509 certificates
            received from the server. This attribute was added in vSphere API
            8.0.0.1.
        """
        self.server = server
        self.username = username
        self.password = password
        self.certificate_authority_chain = certificate_authority_chain
        VapiStruct.__init__(self)


NSXAdvancedLBConfig._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.networks.edges.NSX_advanced_LB_config', {
        'server': type.ReferenceType(__name__, 'Server'),
        'username': type.StringType(),
        'password': type.SecretType(),
        'certificate_authority_chain': type.StringType(),
    },
    NSXAdvancedLBConfig,
    False,
    None))



class HAProxyConfig(VapiStruct):
    """
    ``HAProxyConfig`` class describes configuration for the HAProxy Load
    Balancer. This class was added in vSphere API 8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 servers=None,
                 username=None,
                 password=None,
                 certificate_authority_chain=None,
                ):
        """
        :type  servers: :class:`list` of :class:`Server`
        :param servers: :attr:`HAProxyConfig.servers` is a list of the addresses for the
            data plane API servers used to configure Virtual Servers. This
            attribute was added in vSphere API 8.0.0.1.
        :type  username: :class:`str`
        :param username: :attr:`HAProxyConfig.username` is used by the HAProxy Kubernetes
            Operator to program the HAProxy Controller. This attribute was
            added in vSphere API 8.0.0.1.
        :type  password: :class:`str`
        :param password: :attr:`HAProxyConfig.password` secures the
            :attr:`HAProxyConfig.username`. This attribute was added in vSphere
            API 8.0.0.1.
        :type  certificate_authority_chain: :class:`str`
        :param certificate_authority_chain: :attr:`HAProxyConfig.certificate_authority_chain` contains
            PEM-encoded CA chain which is used to verify x509 certificates
            received from the server. This attribute was added in vSphere API
            8.0.0.1.
        """
        self.servers = servers
        self.username = username
        self.password = password
        self.certificate_authority_chain = certificate_authority_chain
        VapiStruct.__init__(self)


HAProxyConfig._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.networks.edges.HA_proxy_config', {
        'servers': type.ListType(type.ReferenceType(__name__, 'Server')),
        'username': type.StringType(),
        'password': type.SecretType(),
        'certificate_authority_chain': type.StringType(),
    },
    HAProxyConfig,
    False,
    None))



class NSXConfig(VapiStruct):
    """
    ``NSXConfig`` class describes the configuration for NSX-T Edge services.
    This class was added in vSphere API 8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """

    _validator_list = [
        UnionValidator(
            'routing_mode',
            {
                'NAT' : [('egress_IP_ranges', True)],
                'ROUTED' : [],
            }
        ),
    ]


    _canonical_to_pep_names = {
                            'edge_cluster_ID': 'edge_cluster_id',
                            'default_ingress_TLS_certificate': 'default_ingress_tls_certificate',
                            'egress_IP_ranges': 'egress_ip_ranges',
                            }

    def __init__(self,
                 edge_cluster_id=None,
                 default_ingress_tls_certificate=None,
                 routing_mode=None,
                 egress_ip_ranges=None,
                 t0_gateway=None,
                 load_balancer_size=None,
                ):
        """
        :type  edge_cluster_id: :class:`str` or ``None``
        :param edge_cluster_id: :attr:`NSXConfig.edge_cluster_id` defines the NSX Edge Cluster to
            be used for Kubernetes Services of type LoadBalancer, Kubernetes
            Ingresses, and NSX SNAT. This attribute was added in vSphere API
            8.0.0.1.
            Defaults to a the edge cluster created earliest.
        :type  default_ingress_tls_certificate: :class:`str` or ``None``
        :param default_ingress_tls_certificate: :attr:`NSXConfig.default_ingress_tls_certificate` defines a default
            certificate that is served on Ingress services, when another
            certificate is not presented. This configuration applies to all
            namespaces by default. This attribute was added in vSphere API
            8.0.0.1.
            If unset, there will be no certificate served on Ingress.
        :type  routing_mode: :class:`NSXRoutingMode` or ``None``
        :param routing_mode: :attr:`NSXConfig.routing_mode` enables the network topology in
            either NAT mode or Routed Mode. Enabling routed mode will result in
            all the workloads i.e vSphere PODs, VMs and Tanzu clusters Node IPs
            to be directly accessible from networks beyond the Tier-0 router.
            Once a namespace mode is applied, it cannot be changed. This
            attribute was added in vSphere API 8.0.0.1.
            The default setting is NAT mode.
        :type  egress_ip_ranges: :class:`list` of :class:`com.vmware.vcenter.namespace_management.networks_client.IPRange`
        :param egress_ip_ranges: :attr:`NSXConfig.egress_ip_ranges` lists the IP Ranges from which
            NSX assigns IP addresses used for performing SNAT from container
            IPs to external IPs. These ranges must not overlap with other IP
            ranges on this network. This attribute was added in vSphere API
            8.0.0.1.
            If this range is empty, network traffic will not be able to exit
            the cluster.
        :type  t0_gateway: :class:`str` or ``None``
        :param t0_gateway: :attr:`NSXConfig.t0_gateway` specifies the default Tier-0 gateway
            ID for the namespaces configuration. This attribute was added in
            vSphere API 8.0.0.1.
            When clients pass a value of this class as a parameter, the
            attribute must be an identifier for the resource type:
            ``NSXTier0Gateway``. When methods return a value of this class as a
            return value, the attribute will be an identifier for the resource
            type: ``NSXTier0Gateway``.
            If unset, the default Tier-0 gateway from the edge cluster is used.
        :type  load_balancer_size: :class:`LoadBalancerSize` or ``None``
        :param load_balancer_size: :attr:`NSXConfig.load_balancer_size` describes the load balancer
            sizing options available. Larger sizes support more active virtual
            servers, but consume more resources. This attribute was added in
            vSphere API 8.0.0.1.
            If unset, the size defaults to small.
        """
        self.edge_cluster_id = edge_cluster_id
        self.default_ingress_tls_certificate = default_ingress_tls_certificate
        self.routing_mode = routing_mode
        self.egress_ip_ranges = egress_ip_ranges
        self.t0_gateway = t0_gateway
        self.load_balancer_size = load_balancer_size
        VapiStruct.__init__(self)


NSXConfig._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.networks.edges.NSX_config', {
        'edge_cluster_ID': type.OptionalType(type.StringType()),
        'default_ingress_TLS_certificate': type.OptionalType(type.StringType()),
        'routing_mode': type.OptionalType(type.ReferenceType(__name__, 'NSXRoutingMode')),
        'egress_IP_ranges': type.OptionalType(type.ListType(type.ReferenceType('com.vmware.vcenter.namespace_management.networks_client', 'IPRange'))),
        't0_gateway': type.OptionalType(type.IdType()),
        'load_balancer_size': type.OptionalType(type.ReferenceType(__name__, 'LoadBalancerSize')),
    },
    NSXConfig,
    False,
    None))



class Server(VapiStruct):
    """
    A ``Server`` class represents an endpoint used to configure load balancers.
    This class was added in vSphere API 8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 host=None,
                 port=None,
                ):
        """
        :type  host: :class:`str`
        :param host: :attr:`Server.host` specifies a the management hostname or IPv4
            address for a load balancer. This attribute was added in vSphere
            API 8.0.0.1.
        :type  port: :class:`long`
        :param port: :attr:`Server.port` specifies a management port used to access a
            load balancer. This attribute was added in vSphere API 8.0.0.1.
        """
        self.host = host
        self.port = port
        VapiStruct.__init__(self)


Server._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.networks.edges.server', {
        'host': type.StringType(),
        'port': type.IntegerType(),
    },
    Server,
    False,
    None))




class StubFactory(StubFactoryBase):
    _attrs = {
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.lcm.discovery.
#---------------------------------------------------------------------------

"""
The ``com.vmware.vcenter.lcm.discovery_client`` module provides classes for
discovering products registered with vCenter Server and interoperability
between those products and vCenter Server.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from com.vmware.cis_client import Tasks
from vmware.vapi.stdlib.client.task import Task
from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Product(VapiStruct):
    """
    The ``Info`` class contains information about a VMware product which is
    present in the customer Environemnt. The following information about the
    products are present: 
    
    * Name
    * Version
    * Deployments
    * Automatically Discovered or Manually Added

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 installed_product=None,
                 name=None,
                 version=None,
                 target_version=None,
                 deployments=None,
                 auto=None,
                ):
        """
        :type  installed_product: :class:`str`
        :param installed_product: Identifies a product and a version uniquely. 
            
            The identifier consists of product internal name and version.
            When clients pass a value of this class as a parameter, the
            attribute must be an identifier for the resource type: ``PRODUCT``.
            When methods return a value of this class as a return value, the
            attribute will be an identifier for the resource type: ``PRODUCT``.
        :type  name: :class:`str`
        :param name: A public official product name.
        :type  version: :class:`str`
        :param version: Current product version.
        :type  target_version: :class:`str` or ``None``
        :param target_version: Future version of the product after upgrade.
            ``targetVersion`` may not be applicable.
        :type  deployments: :class:`list` of :class:`str` or ``None``
        :param deployments: The list of hostname/IPs of the instances of the VMware products
            deployed in the environment. This field would be empty for manually
            added products.
        :type  auto: :class:`bool`
        :param auto: Indicates if the product is auto-detected by the system or manually
            added. If it is set to true it means it is auto-detected.
        """
        self.installed_product = installed_product
        self.name = name
        self.version = version
        self.target_version = target_version
        self.deployments = deployments
        self.auto = auto
        VapiStruct.__init__(self)


Product._set_binding_type(type.StructType(
    'com.vmware.vcenter.lcm.discovery.product', {
        'installed_product': type.IdType(resource_types='PRODUCT'),
        'name': type.StringType(),
        'version': type.StringType(),
        'target_version': type.OptionalType(type.StringType()),
        'deployments': type.OptionalType(type.ListType(type.StringType())),
        'auto': type.BooleanType(),
    },
    Product,
    False,
    None))



class InteropReport(VapiInterface):
    """
    The ``InteropReport`` interface provides methods to report the
    interoperability between a vCenter Server release version and the other
    installed VMware products registered in the vCenter Server instance.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.lcm.discovery.interop_report'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _InteropReportStub)
        self._VAPI_OPERATION_IDS = {}
        self._VAPI_OPERATION_IDS.update({'create_task': 'create$task'})

    class ReleaseInfo(VapiStruct):
        """
        The ``InteropReport.ReleaseInfo`` class contains a product release
        information.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     version=None,
                     note=None,
                    ):
            """
            :type  version: :class:`str`
            :param version: The version of the release.
            :type  note: :class:`str` or ``None``
            :param note: A link to the release notes of the release.
                None if the release notes are not available.
            """
            self.version = version
            self.note = note
            VapiStruct.__init__(self)


    ReleaseInfo._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.interop_report.release_info', {
            'version': type.StringType(),
            'note': type.OptionalType(type.URIType()),
        },
        ReleaseInfo,
        False,
        None))


    class ReportRow(VapiStruct):
        """
        The ``InteropReport.ReportRow`` class contains the interoperability between
        a given product and the target product.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     product=None,
                     compatible=None,
                     compatible_releases=None,
                    ):
            """
            :type  product: :class:`Product`
            :param product: The product to compare to the target product.
            :type  compatible: :class:`bool`
            :param compatible: Defines whether the product is compatible against the target
                product.
            :type  compatible_releases: :class:`list` of :class:`InteropReport.ReleaseInfo`
            :param compatible_releases: A list of compatible releases of the product with the target
                product.
            """
            self.product = product
            self.compatible = compatible
            self.compatible_releases = compatible_releases
            VapiStruct.__init__(self)


    ReportRow._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.interop_report.report_row', {
            'product': type.ReferenceType(__name__, 'Product'),
            'compatible': type.BooleanType(),
            'compatible_releases': type.ListType(type.ReferenceType(__name__, 'InteropReport.ReleaseInfo')),
        },
        ReportRow,
        False,
        None))


    class ReportSummary(VapiStruct):
        """
        The ``InteropReport.ReportSummary`` class contains a summary of the
        :attr:`InteropReport.Report.products`. It consists of the count of
        compatible and incompatible products to the target product.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     compatible_count=None,
                     incompatible_count=None,
                    ):
            """
            :type  compatible_count: :class:`long`
            :param compatible_count: Number of compatible products.
            :type  incompatible_count: :class:`long`
            :param incompatible_count: Number of incompatible products.
            """
            self.compatible_count = compatible_count
            self.incompatible_count = incompatible_count
            VapiStruct.__init__(self)


    ReportSummary._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.interop_report.report_summary', {
            'compatible_count': type.IntegerType(),
            'incompatible_count': type.IntegerType(),
        },
        ReportSummary,
        False,
        None))


    class Report(VapiStruct):
        """
        The ``InteropReport.Report`` class contains the interoperability report
        between the target product and the other registered products in the vCenter
        Server instance.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     date_created=None,
                     target_product=None,
                     products=None,
                     issues=None,
                     summary=None,
                    ):
            """
            :type  date_created: :class:`datetime.datetime`
            :param date_created: Time when the report is created.
            :type  target_product: :class:`Product`
            :param target_product: A product against the other products are compared to. Only vCenter
                Server is supported.
            :type  products: :class:`list` of :class:`InteropReport.ReportRow`
            :param products: Interoperability matrix for the supplied target product and the
                other registered products.
            :type  issues: :class:`com.vmware.vcenter.lcm_client.Notifications` or ``None``
            :param issues: Lists of issues encountered during report creation.
                :class:`set` if any issues encountered.
            :type  summary: :class:`InteropReport.ReportSummary`
            :param summary: A summary of the interoperability matrix.
            """
            self.date_created = date_created
            self.target_product = target_product
            self.products = products
            self.issues = issues
            self.summary = summary
            VapiStruct.__init__(self)


    Report._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.interop_report.report', {
            'date_created': type.DateTimeType(),
            'target_product': type.ReferenceType(__name__, 'Product'),
            'products': type.ListType(type.ReferenceType(__name__, 'InteropReport.ReportRow')),
            'issues': type.OptionalType(type.ReferenceType('com.vmware.vcenter.lcm_client', 'Notifications')),
            'summary': type.ReferenceType(__name__, 'InteropReport.ReportSummary'),
        },
        Report,
        False,
        None))


    class Result(VapiStruct):
        """
        The ``InteropReport.Result`` class contains the result of interoperability
        report creation operation.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     report=None,
                     csv_report=None,
                    ):
            """
            :type  report: :class:`InteropReport.Report`
            :param report: The interoperability report.
            :type  csv_report: :class:`str` or ``None``
            :param csv_report: The identifier of CSV formatted interopability report. 
                
                com.vmware.vcenter.lcm.report.Report#get provides location where
                the CSV report can be downloaded from based on the ``csvReport``.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.vcenter.lcm.report``. When methods return a value of
                this class as a return value, the attribute will be an identifier
                for the resource type: ``com.vmware.vcenter.lcm.report``.
                None in case of ``errors`` reported in
                :attr:`InteropReport.Report.issues`.
            """
            self.report = report
            self.csv_report = csv_report
            VapiStruct.__init__(self)


    Result._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.interop_report.result', {
            'report': type.ReferenceType(__name__, 'InteropReport.Report'),
            'csv_report': type.OptionalType(type.IdType()),
        },
        Result,
        False,
        None))


    class Spec(VapiStruct):
        """
        Configuration of report generation.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     target_version=None,
                    ):
            """
            :type  target_version: :class:`str`
            :param target_version: The vCenter Server version. 
                
                It is used for checking against the other products registered with
                that instance of vCenter Server.
            """
            self.target_version = target_version
            VapiStruct.__init__(self)


    Spec._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.interop_report.spec', {
            'target_version': type.StringType(),
        },
        Spec,
        False,
        None))




    def create_task(self,
               spec=None,
               ):
        """
        Creates interoperability report between a vCenter Server release
        version and all registered products with the vCenter Server instance. 
        
        The result of this operation can be queried by calling the
        com.vmware.cis.Tasks#get method where ``task`` is the response of this
        operation.

        :type  spec: :class:`InteropReport.Spec` or ``None``
        :param spec: 
            Specifies the target version against this interoperability check
            report will be generated. If None the report will be generated for
            the currently installed version of the vCenter server.
        :rtype: :class:  `vmware.vapi.stdlib.client.task.Task`
        :return: Task instance
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If there is some unknown internal error. The accompanying error
            message will give more details about the failure.
        """
        task_id = self._invoke('create$task',
                                {
                                'spec': spec,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.ReferenceType(__name__, 'InteropReport.Result'))
        return task_instance
class ProductCatalog(VapiInterface):
    """
    The ``ProductCatalog`` class provides information which VMware products can
    be associated with vCenter Server.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.lcm.discovery.product_catalog'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _ProductCatalogStub)
        self._VAPI_OPERATION_IDS = {}

    class Summary(VapiStruct):
        """
        The ``ProductCatalog.Summary`` class contains information about each VMware
        product and its corresponding versions that can be associated with vCenter
        Server.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     name=None,
                     versions=None,
                    ):
            """
            :type  name: :class:`str`
            :param name: A product name that the customer is aware of.
            :type  versions: :class:`list` of :class:`str`
            :param versions: List of versions the customer can select from.
            """
            self.name = name
            self.versions = versions
            VapiStruct.__init__(self)


    Summary._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.product_catalog.summary', {
            'name': type.StringType(),
            'versions': type.ListType(type.StringType()),
        },
        Summary,
        False,
        None))



    def list(self):
        """
        Retrieves a list of all VMware products that can be associated with
        vCenter Server.


        :rtype: :class:`list` of :class:`ProductCatalog.Summary`
        :return: List of all the VMware products which can be associated with
            vCenter Server
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If there is some unknown internal error. The accompanying error
            message will give more details about the failure.
        """
        return self._invoke('list', None)
class AssociatedProducts(VapiInterface):
    """
    The ``AssociatedProducts`` class provides options to list, add, modify, and
    delete VMware products associated with vCenter Server. Some products can be
    auto-detected by the system while others can be added manually.
    """
    RESOURCE_TYPE = "com.vmware.vcenter.lcm.product"
    """
    The resource type for the products interface.

    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.lcm.discovery.associated_products'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _AssociatedProductsStub)
        self._VAPI_OPERATION_IDS = {}

    class CreateSpec(VapiStruct):
        """
        The ``AssociatedProducts.CreateSpec`` class is the specification used for
        the product creation.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     product_name=None,
                     version=None,
                     deployments=None,
                    ):
            """
            :type  product_name: :class:`str`
            :param product_name: The name of the product.
            :type  version: :class:`str`
            :param version: Current product version.
            :type  deployments: :class:`list` of :class:`str` or ``None``
            :param deployments: The list of hostname/IPs of the instances of the VMware products
                deployed in the environment.
            """
            self.product_name = product_name
            self.version = version
            self.deployments = deployments
            VapiStruct.__init__(self)


    CreateSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.associated_products.create_spec', {
            'product_name': type.StringType(),
            'version': type.StringType(),
            'deployments': type.OptionalType(type.ListType(type.StringType())),
        },
        CreateSpec,
        False,
        None))


    class UpdateSpec(VapiStruct):
        """
        The ``AssociatedProducts.UpdateSpec`` class is the specification for the
        product update.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     deployments=None,
                    ):
            """
            :type  deployments: :class:`list` of :class:`str` or ``None``
            :param deployments: The list of hostname/IPs of the instances of the VMware products
                deployed in the environment.
            """
            self.deployments = deployments
            VapiStruct.__init__(self)


    UpdateSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.lcm.discovery.associated_products.update_spec', {
            'deployments': type.OptionalType(type.ListType(type.StringType())),
        },
        UpdateSpec,
        False,
        None))



    def list(self):
        """
        Retrieves a list of all associated VMware product deployments with
        vCenter Server in the environment. The list contains both product
        deployments discovered automatically and deployments registered
        manually through the API.


        :rtype: :class:`list` of :class:`Product`
        :return: List of all the registered products with vCenter.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If there is some unknown internal error. The accompanying error
            message will give more details about the failure.
        """
        return self._invoke('list', None)

    def create(self,
               spec,
               ):
        """
        Associates a VMware product with vCenter Server manually. The product
        must be taken from the product catalog API.

        :type  spec: :class:`AssociatedProducts.CreateSpec`
        :param spec: Info creation specification.
        :rtype: :class:`str`
        :return: Identifier of the newly-added product.
            The return value will be an identifier for the resource type:
            ``com.vmware.vcenter.lcm.product``.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.AlreadyExists` 
            if this version is already added
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            if the spec argument is not allowed
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If there is some unknown internal error. The accompanying error
            message will give more details about the failure.
        """
        return self._invoke('create',
                            {
                            'spec': spec,
                            })

    def get(self,
            product,
            ):
        """
        Returns the detailed information of a product associated with vCenter
        Server.

        :type  product: :class:`str`
        :param product: An identifier of the product to be modified.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.lcm.product``.
        :rtype: :class:`Product`
        :return: Product details.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If there is no record associated with ``product`` in the system.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If there is some unknown internal error. The accompanying error
            message will give more details about the failure.
        """
        return self._invoke('get',
                            {
                            'product': product,
                            })

    def update(self,
               product,
               spec,
               ):
        """
        Modifies a VMware product associated with vCenter Server which was
        added manually. Automatically discovered VMware products cannot be
        modified.

        :type  product: :class:`str`
        :param product: An id of the product to be modified.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.lcm.product``.
        :type  spec: :class:`AssociatedProducts.UpdateSpec`
        :param spec: 
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            if the spec argument is not allowed
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If there is no record associated with ``product`` in the system.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If there is some unknown internal error. The accompanying error
            message will give more details about the failure.
        """
        return self._invoke('update',
                            {
                            'product': product,
                            'spec': spec,
                            })

    def delete(self,
               product,
               ):
        """
        Deletes or dissociates a VMware product associated with vCenter Server
        which was added manually. Automatically discovered VMware products
        cannot be deleted or dissociated.

        :type  product: :class:`str`
        :param product: An id of the product to be removed.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.lcm.product``.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If there is no record associated with ``product`` in the system
            database.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If there is some unknown internal error. The accompanying error
            message will give more details about the failure.
        """
        return self._invoke('delete',
                            {
                            'product': product,
                            })
class _InteropReportStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for create operation
        create_input_type = type.StructType('operation-input', {
            'spec': type.OptionalType(type.ReferenceType(__name__, 'InteropReport.Spec')),
        })
        create_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        create_input_value_validator_list = [
        ]
        create_output_validator_list = [
        ]
        create_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/lcm/discovery/interop-report',
            request_body_parameter='spec',
            path_variables={
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        operations = {
            'create$task': {
                'input_type': create_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': create_error_dict,
                'input_value_validator_list': create_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
        }
        rest_metadata = {
            'create': create_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.lcm.discovery.interop_report',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)

class _ProductCatalogStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {})
        list_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/lcm/discovery/product-catalog',
            path_variables={
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ListType(type.ReferenceType(__name__, 'ProductCatalog.Summary')),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.lcm.discovery.product_catalog',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)

class _AssociatedProductsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {})
        list_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/lcm/discovery/associated-products',
            path_variables={
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for create operation
        create_input_type = type.StructType('operation-input', {
            'spec': type.ReferenceType(__name__, 'AssociatedProducts.CreateSpec'),
        })
        create_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.already_exists':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'AlreadyExists'),
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        create_input_value_validator_list = [
        ]
        create_output_validator_list = [
        ]
        create_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/lcm/discovery/associated-products',
            request_body_parameter='spec',
            path_variables={
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'product': type.IdType(resource_types='com.vmware.vcenter.lcm.product'),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/lcm/discovery/associated-products/{product}',
            path_variables={
                'product': 'product',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for update operation
        update_input_type = type.StructType('operation-input', {
            'product': type.IdType(resource_types='com.vmware.vcenter.lcm.product'),
            'spec': type.ReferenceType(__name__, 'AssociatedProducts.UpdateSpec'),
        })
        update_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        update_input_value_validator_list = [
        ]
        update_output_validator_list = [
        ]
        update_rest_metadata = OperationRestMetadata(
            http_method='PATCH',
            url_template='/vcenter/lcm/discovery/associated-products/{product}',
            request_body_parameter='spec',
            path_variables={
                'product': 'product',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for delete operation
        delete_input_type = type.StructType('operation-input', {
            'product': type.IdType(resource_types='com.vmware.vcenter.lcm.product'),
        })
        delete_error_dict = {
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        delete_input_value_validator_list = [
        ]
        delete_output_validator_list = [
        ]
        delete_rest_metadata = OperationRestMetadata(
            http_method='DELETE',
            url_template='/vcenter/lcm/discovery/associated-products/{product}',
            path_variables={
                'product': 'product',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ListType(type.ReferenceType(__name__, 'Product')),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'create': {
                'input_type': create_input_type,
                'output_type': type.IdType(resource_types='com.vmware.vcenter.lcm.product'),
                'errors': create_error_dict,
                'input_value_validator_list': create_input_value_validator_list,
                'output_validator_list': create_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType(__name__, 'Product'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'update': {
                'input_type': update_input_type,
                'output_type': type.VoidType(),
                'errors': update_error_dict,
                'input_value_validator_list': update_input_value_validator_list,
                'output_validator_list': update_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'delete': {
                'input_type': delete_input_type,
                'output_type': type.VoidType(),
                'errors': delete_error_dict,
                'input_value_validator_list': delete_input_value_validator_list,
                'output_validator_list': delete_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
            'create': create_rest_metadata,
            'get': get_rest_metadata,
            'update': update_rest_metadata,
            'delete': delete_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.lcm.discovery.associated_products',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'InteropReport': InteropReport,
        'ProductCatalog': ProductCatalog,
        'AssociatedProducts': AssociatedProducts,
    }




"""
Helper classes for client side localization.
"""
__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long


from vmware.vapi.l10n.formatter import StringFormatter


class MessageLocalizer(object):
    """
    Helper class to localize vAPI LocalizableMessages.

    Before initializating the MessageLocalizer, locale
    has to be explicitly set if the user wants a locale
    other than English (U.S.).

    Typically, at the start of the application, the following
    two lines are required to set the locale.

    import locale
    locale.setlocale(locale.LC_ALL,<locale_type>)

    If the locale_type is empty string, locale is set to user's
    default setting (typically specified in the LANG environment
    variable). Otherwise, the specified locale type has to be
    installed in the system.
    """
    def __init__(self, bundle, formatter=StringFormatter):
        """
        Initialize MessageLocalizer

        :type  bundle: :class:`vmware.vapi.message.MessageBundle`
        :param bundle: Message bundle to retrieve the templates for the
            messages
        :type  formatter: :class:`vmware.vapi.message.MessageFormatter`
        :param formatter: Message formatter to format the messages
        """
        self._bundle = bundle
        self._formatter = formatter

    def localize(self, msg):
        """
        Localize a given LocalizableMessage.

        :type  msg: :class:`com.vmware.vapi.std_client.LocalizableMessage`
        :param msg: LocalizableMessage
        :rtype: :class:`str`
        :return: Localized string
        """
        try:
            template = self._bundle.get(msg.id)
        except KeyError:
            raise Exception('Invalid message identifier: %s' % msg.id)
        return self._formatter.format_msg(template, msg.args)
